/******************   BiDirectional Associative Memory   ****************/
/*	  BAMGRAPH.C							*/
/*	  Written by Kevin L. Priddy					*/
/*        Version 2.0							*/
/*	  20 July 1991							*/
/*									*/
/*        This program makes a set of vectors which represent           */
/*        associative memories.  The vectors are then used to create    */
/*	  a weight matrix which is a transformation from one vector     */
/*	  space to the other vector space.                              */
/************************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <graphics.h>
#include <defines.h>
#include <alloc.h>
extern FILE *open_data_file();
extern setup parameters;
#define seed 218
#define bitchange 4
#define IDX(I,J,N) ((I)*(N)+(J))
#define MALLOCk(A,B,C,D) if((A = (C *)calloc(sizeof(C),(B)))==NULL) { \
    fprintf(stderr, strcat(D,": insufficient memory\n") ); \
    exit (-1); }


int iteration=0;
int Count=0;
int choice = 1;
int choice2 = 1;
int **IMATRIX();
int *IVECTOR();
int freeIMATRIX();
extern int reset;
extern void *buffer;
struct start
  {
  int **A_Matrix, **B_Matrix, **M_Matrix, **A_TestMatrix, **B_TestMatrix;
  int **A_OutTestMatrix, **B_OutTestMatrix, **HamIn, **HamOut;
  int *A_In, *B_In, *A_Out, *B_Out;
  int ANodes, BNodes, Exemplars, ColA, ColB, RowA, RowB;
  }   ;

FILE *open_a_data_file(int bx,int by,char *default_name)
{  char aname[20];
   FILE *afile_pointer;
   strcpy(parameters.dirmask,"*.hop");
   setviewport(bx,by,bx+150,by+100,1);
   clearviewport();
   strcpy(aname,default_name);
   while(TRUE){
	get_file_name(aname);
	if(open_data_file(aname)){afile_pointer=fopen(aname,"r");break;}
	}

return afile_pointer;

}

bam()
{
struct start all;
FILE *fin;
char filename[81],aname[20];
char in_string[81];
int i, j, k;
reset = TRUE;

Init_screen();

fin = open_a_data_file(30,60,"bam.hop");
wipescrn();
fscanf( fin, "%d %d %d %d %d", &all.Exemplars, &all.RowA, &all.ColA,
	&all.RowB, &all.ColB);

all.ANodes = all.RowA*all.ColA;
all.BNodes = all.RowB*all.ColB;

wipescrn();

/********************** Allocate Memory for Arrays **********************/
all.M_Matrix = IMATRIX(all.ANodes, all.BNodes, "all.M_Matrix");
all.A_TestMatrix = IMATRIX(all.Exemplars, all.ANodes, "all.A_TestMatrix");
all.B_TestMatrix = IMATRIX(all.Exemplars, all.BNodes, "all.B_TestMatrix");
all.A_Matrix = IMATRIX(all.Exemplars, all.ANodes, "all.A_Matrix");
all.B_Matrix = IMATRIX(all.Exemplars, all.BNodes, "all.B_Matrix");
all.A_OutTestMatrix = IMATRIX(all.Exemplars, all.ANodes, "all.A_OutTestMatrix");
all.B_OutTestMatrix = IMATRIX(all.Exemplars, all.BNodes, "all.B_OutTestMatrix");
all.B_Out = IVECTOR(all.BNodes, "all.B_Out");
all.A_Out = IVECTOR(all.ANodes, "all.A_Out");
all.B_In = IVECTOR(all.BNodes,  "all.B_In");
all.A_In = IVECTOR(all.ANodes,  "all.A_In");
all.HamIn = IMATRIX(all.Exemplars, all.Exemplars, "all.HamIn");
all.HamOut = IMATRIX(all.Exemplars, all.Exemplars, "all.HamOut");

/************* Read Data for A_Matrix and B_Matrix **********************/
loopi(all.Exemplars)
   {
   loopj(all.ANodes)
      fscanf(fin,"%d",&all.A_Matrix[i][j]);
   loopk(all.BNodes)
      fscanf(fin,"%d",&all.B_Matrix[i][k]);
   }

M_Bipolar(all.A_Matrix, all.A_Matrix, all.Exemplars, all.ANodes);
M_Bipolar(all.B_Matrix, all.B_Matrix, all.Exemplars, all.BNodes);
Compute_M(all.A_Matrix,all.B_Matrix, all.M_Matrix, all.ANodes, all.BNodes, all.Exemplars);

Get_Started(all);

M_Unipolar(all.A_Matrix, all.A_Matrix, all.Exemplars, all.ANodes);
M_Unipolar(all.B_OutTestMatrix, all.B_OutTestMatrix, all.Exemplars, all.BNodes);
M_Unipolar(all.B_Matrix, all.B_Matrix, all.Exemplars, all.BNodes);
M_Unipolar(all.A_OutTestMatrix, all.A_OutTestMatrix, all.Exemplars, all.ANodes);

Results(all);

free(all.A_In);
free(all.A_Out);
free(all.B_In);
free(all.B_Out);
freeIMATRIX(all.ANodes, all.M_Matrix);
freeIMATRIX(all.Exemplars, all.A_TestMatrix);
freeIMATRIX(all.Exemplars, all.B_TestMatrix);
freeIMATRIX(all.Exemplars, all.A_Matrix);
freeIMATRIX(all.Exemplars, all.B_Matrix);
freeIMATRIX(all.Exemplars, all.A_OutTestMatrix);
freeIMATRIX(all.Exemplars, all.B_OutTestMatrix);
freeIMATRIX(all.Exemplars, all.HamIn);
freeIMATRIX(all.Exemplars, all.HamOut);


fclose(fin);

}        /************* End main() **************/


Init_screen()
{
make_screen();
wipescrn();
}
/************************ Function Definitions ************************

/************************** wipescrn() ******************************/
wipescrn()
   {
		 /** clears text screen **/
clrscr();
setviewport(0,0,639,349,1);
setbkcolor(BLUE);
rectangle(20,50,600,300);
setcolor(WHITE);
settextstyle(1,0,2);
outtextxy(150,10,"Bi Directional Associative Memory ");
   }
/*************************** End wipescrn() ***************************/

/************************ Compute_APrime() ******************************/
Compute_APrime(int *B_In, int *A_Out, int **M_Matrix,
					   int ANodes, int BNodes)
   {
   int i, j;

      loopij(ANodes,BNodes)
	    A_Out[i] += B_In[j] * M_Matrix[i][j];
   V_Bipolar(A_Out, A_Out, ANodes);
   }

/************************** End Compute_APrime() ************************/

/************************ Compute_BPrime() ******************************/
Compute_BPrime(int *A_In, int *B_Out, int **M_Matrix,
					   int ANodes, int BNodes)
   {
   int i, j;
      loopij(ANodes,BNodes)
	    B_Out[j] += A_In[i] * M_Matrix[i][j];
      V_Bipolar(B_Out, B_Out, BNodes);
   }

/************************** End Compute_BPrime() ************************/

/************************* Compute_M() ************************************/
Compute_M(int **A_Matrix, int **B_Matrix, int **M_Matrix,
				    int ANodes, int BNodes, int Exemplars)
{
int i, j, k;
loopij(ANodes,BNodes)
  M_Matrix[i][j] = 0;
loopij(ANodes,BNodes)
  loopk(Exemplars)
    M_Matrix[i][j] +=
		    A_Matrix[k][i] * B_Matrix[k][j];
}
/************************  End Compute_M *********************************/

/************************ M_Bipolar() *******************************/
M_Bipolar(int **inarray, int **outarray, int icntr, int jcntr)
   {
   int i,j;
   loopi(icntr)
      loopj(jcntr)
	 outarray[i][j] = (inarray[i][j] <= 0) ? -1 : 1;
   }
/************************* End M_Bipolar() **************************/

/************************ M_Unipolar() *******************************/
M_Unipolar(int **inarray, int **outarray, int icntr, int jcntr)
   {
   int i,j;
   loopi(icntr)
      loopj(jcntr)
	 outarray[i][j] = (inarray[i][j] <= 0) ? 0 : 1;
   }
/************************* End M_Unipolar() **************************/


/************************ V_Bipolar() *******************************/
V_Bipolar(int *invector, int *outvector, int icntr)
   {
   int i,j;
   loopi(icntr)
	 outvector[i] = (invector[i] <= 0) ? -1 : 1;
   }
/************************* End V_Bipolar() **************************/

/************************ V_Unipolar() *******************************/
V_Unipolar(int *invector, int *outvector, int icntr)
   {
   int i,j;
   loopi(icntr)
	 outvector[i] = (invector[i] <= 0) ? 0 : 1;
   }
/************************* End V_Unipolar() **************************/


/*************************** ShowMatrix_backup() *******************************/
ShowMatrix_backup(int **inarray, int icntr, int jcntr, int kcntr, char *title)
   {
   int i, j, k;
   printf("\n                  %s\n\n",title);
   loopi(icntr)
      {

      loopj(jcntr)
	{
	printf("%3d",inarray[i][j]);
	if (((j+1) % kcntr) == 0)
	   printf("\n");
	}
      printf("\n");
      }
   }
/*************************  End Of ShowMatrix() ***********************/

/*************************** ShowMatrix() *******************************/
ShowMatrix(int **inarray, int xpos, int ypos, int icntr, int jcntr,
		int kcntr)
{
   int i=icntr, j, k, x=xpos, y=ypos, wide=8;
      loopj(jcntr)
	{
	 x=xpos+(j % kcntr)*wide;
	if ((j % kcntr) == 0) y=y+wide/2;
	setfillstyle(SOLID_FILL,inarray[i][j]+3);
	bar(x,y,x+wide,y+wide/2);
	}
}
/*************************  End Of ShowMatrix() ***********************/


/*************************** PrintMatrix *********************************/
PrintMatrix(int **inarray, int icntr, int jcntr, int kcntr, char *title)
   {
   int i, j, k;
   fprintf(stdprn,"\n                  %s\n\n",title);
   loopi(icntr)
     {
     loopj(jcntr)
	{
	fprintf(stdprn,"%3d",inarray[i][j]);
	if (((j+1) % kcntr) == 0)
	   fprintf(stdprn,"\n");
	}
     fprintf(stdprn,"\n");
     }
   fprintf(stdprn,"\n");
   }
	      /************  End Of PrintMatrix ****************/

/*************************** Get_Started() ******************************/
Get_Started(struct start all)
   {
   int i, j, k;
   gotoxy(5,5);
   printf("Press the key which you desire and hit Return");
   gotoxy(5,6);
   printf("1: Generate A Vectors");
   gotoxy(5,7);
   printf("2: Generate B Vectors");
   gotoxy(5,8);
   printf("Your Choice: ");
   scanf("%d", &choice);
   wipescrn();
   switch (choice)
      {
      case 1 :
	 Make_Test(all.A_Matrix, all.A_TestMatrix, all.Exemplars,
		   all.ANodes, all.Exemplars);
	 CalcHamming(all.A_TestMatrix, all.A_Matrix, all.HamIn,
		     all.Exemplars, all.Exemplars, all.ANodes);
	 loopi(all.Exemplars)
	 {
	 Count = 0;
	 iteration = 0;
	 loopj(all.ANodes)
	    all.A_In[j] = all.A_TestMatrix[i][j];
	 Find_Solution(all);
	 loopk(all.BNodes)
	    all.B_OutTestMatrix[i][k] = all.B_Out[k];
	 loopj(all.ANodes)
	    all.A_OutTestMatrix[i][j] = all.A_Out[k];
	 ShowCalc(all,i+1);
	 if (i == (all.Exemplars-1))
	    {
            gotoxy(15,24);
	    printf("***       Press Spacebar to Continue      ***");
	    getche();
	    }
	 }
	 CalcHamming(all.B_OutTestMatrix, all.B_Matrix, all.HamOut,
		     all.Exemplars, all.Exemplars, all.BNodes);
	 break;
      case 2 :
	 Make_Test(all.B_Matrix, all.B_TestMatrix, all.Exemplars,
		   all.BNodes, all.Exemplars);
	 CalcHamming(all.B_TestMatrix, all.B_Matrix, all.HamIn,
		     all.Exemplars, all.Exemplars, all.BNodes);
	 loopi(all.Exemplars)
	 {
	 Count = 0;
	 iteration = 0;
	 loopj(all.BNodes)
	    all.B_In[j] = all.B_TestMatrix[i][j];
	 Find_Solution(all,i);
	 loopk(all.ANodes)
	    all.A_OutTestMatrix[i][k] = all.A_Out[k];
         loopk(all.BNodes)
	    all.B_OutTestMatrix[i][k] = all.B_Out[k];
	 ShowCalc(all,i+1);
	 if (i == (all.Exemplars-1))
	    {
            gotoxy(15,24);
	    printf("***       Press Spacebar to Continue      ***");
	    getche();
	    }
	 }
	 CalcHamming(all.A_OutTestMatrix, all.A_Matrix, all.HamOut,
		     all.Exemplars, all.Exemplars, all.ANodes);
	 break;
      }
   }
/***************************** End Get_Started *************************/

/***************************** Make_Test() ****************************/
Make_Test(int **inarray, int **outarray, int icntr, int jcntr, int Exemplars)
{
int i, j, temp;                 /* Make Bipolar Test Matrices */
loopi(icntr)
   {
   temp = (i + 1) % Exemplars;
   loopj(jcntr)
      {
      outarray[i][j] = inarray[temp][j];
      if ( rand() < RAND_MAX/bitchange)
	 outarray[i][j] = -outarray[i][j];
      }
   }
}

/************************* End Make_Test() **************************/

/******************** CheckOutput() ******************************/
CheckOutput(int *input, int *output, int icntr)
{
   int i, Sum=0;
   loopi(icntr)
      {
      if (output[i] == input[i])
	 Sum++;
      else
	 Sum = 0;
      }
      if (Sum == icntr)
	 Count++;
      else
	 UpdateInput(input, output, icntr);
   }      /**************** End CheckOutput *****************/

/*********************** UpdateInput() ***********************************/
UpdateInput(int *input, int *output, int icntr)
   {
   int i;
   Count = 0;
   loopi(icntr)
      input[i] = output[i];
   }
/************************** End UpdateInput *****************************/

/************************** Find_Solution() *****************************/
Find_Solution(struct start all)
   {

   Count = 0;
   while (Count < 3)
      {
      switch (choice)
	 {
	 case 1 :

	    CheckOutput(all.B_In, all.B_Out, all.BNodes);
	    Compute_APrime(all.B_In, all.A_Out, all.M_Matrix, all.ANodes, all.BNodes);
	    Compute_BPrime(all.A_In, all.B_Out, all.M_Matrix, all.ANodes, all.BNodes);
	    break;
	 case 2 :
	    CheckOutput(all.A_In, all.A_Out, all.ANodes);
	    Compute_BPrime(all.A_In, all.B_Out, all.M_Matrix, all.ANodes, all.BNodes);
	    Compute_APrime(all.B_In, all.A_Out, all.M_Matrix, all.ANodes, all.BNodes);
	    break;
	 }
      }
   }
/**************************** End Find_Solution() ************************/

/**************************** ShowResults() *****************************/
ShowResults(struct start all)
{
int i;
switch (choice)
  {
  case 1 :
    wipescrn();
    loopi(all.Exemplars)
       {
       ShowMatrix(all.A_Matrix, 100, 50*(i+1), i, all.ANodes, all.ColA);
       ShowMatrix(all.B_Matrix, 300, 50*(i+1), i, all.BNodes, all.ColB);
       }
    gotoxy(15,24);
    printf("***       Press Spacebar to Continue      ***");
    getche(); wipescrn();
    loopi(all.Exemplars)
       {
       ShowMatrix(all.A_TestMatrix, 100, 50*(i+1), i, all.ANodes, all.ColA);
       ShowMatrix(all.B_OutTestMatrix, 300, 50*(i+1), i, all.BNodes, all.ColB);
       }
    gotoxy(15,24);
    printf("***       Press Spacebar to Continue      ***");
    getche(); wipescrn();
    ShowHamming(all.HamIn, all.Exemplars, all.Exemplars, 0, "The Input Test Hamming Matrix");
 /*   gotoxy(15,24);
    printf("***       Press Spacebar to Continue      ***");
    getche(); wipescrn(); */
    ShowHamming(all.HamOut, all.Exemplars, all.Exemplars, 1, "The Output Hamming Matrix");
    gotoxy(15,24);
    printf("***       Press Spacebar to Continue      ***");
    getche(); wipescrn();
    break;

  case 2 :
    wipescrn();
    loopi(all.Exemplars)
       {
       ShowMatrix(all.A_Matrix, 100, 50*(i+1), i, all.ANodes, all.ColA);
       ShowMatrix(all.B_Matrix, 300, 50*(i+1), i, all.BNodes, all.ColB);
       }
    gotoxy(15,24);
    printf("***       Press Spacebar to Continue      ***");
    getche(); wipescrn();
    loopi(all.Exemplars)
       {
       ShowMatrix(all.B_TestMatrix, 100, 50*(i+1), i, all.BNodes, all.ColB);
       ShowMatrix(all.A_OutTestMatrix, 300, 50*(i+1), i, all.ANodes,all.ColA);
       }
    gotoxy(15,24);
    printf("***       Press Spacebar to Continue      ***");
    getche(); wipescrn();
    ShowHamming(all.HamIn, all.Exemplars, all.Exemplars, 0, "The Input Test Hamming Matrix");
 /*   gotoxy(15,24);
    printf("***       Press Spacebar to Continue      ***");
    getche(); wipescrn(); */
    ShowHamming(all.HamOut, all.Exemplars, all.Exemplars, 1, "The Output Hamming Matrix");
    gotoxy(15,24);
    printf("***       Press Spacebar to Continue      ***");
    getche(); wipescrn();
  break;
  }
}
/******************************** End ShowResults() *************************/

/**************************** ShowCalc() *****************************/
ShowCalc(struct start all, int icntr)
{
int i;
switch (choice)
  {
  case 1 :
    loopi(icntr)
       {
       ShowMatrix(all.A_TestMatrix, 100, 50*(i+1), i, all.ANodes,
		  all.ColA);
       ShowMatrix(all.B_OutTestMatrix, 300, 50*(i+1), i, all.BNodes,
		  all.ColB);
       }
    break;

  case 2 :
    loopi(icntr)
       {
       ShowMatrix(all.B_TestMatrix, 100, 50*(i+1), i, all.BNodes,
		  all.ColB);
       ShowMatrix(all.A_OutTestMatrix, 300, 50*(i+1), i, all.ANodes,
		  all.ColA);
       }
  break;
  }
}
/******************************** End ShowResults() *************************/


/**************************** PrintResults() *****************************/
PrintResults(struct start all)
{
fprintf(stdprn,"Seed = %5d   Bitchange = %2d",seed, bitchange);
switch (choice)
  {
  case 1 :
    PrintMatrix(all.A_Matrix, all.Exemplars, all.ANodes, all.ColA,
		"The A Exemplar Matrix");
    PrintMatrix(all.B_Matrix, all.Exemplars, all.BNodes, all.ColB,
				    "The Corresponding B Exemplar Matrix");
    PrintMatrix(all.A_TestMatrix, all.Exemplars, all.ANodes, all.ColA,
		"The Input Test Vector Matrix");
    PrintMatrix(all.B_OutTestMatrix, all.Exemplars, all.BNodes, all.ColB,
		"The Output Matrix");
    PrintHamming(all.HamIn, all.Exemplars, all.Exemplars,
		 "The Input Test Hamming Matrix");
    PrintHamming(all.HamOut, all.Exemplars, all.Exemplars,
		 "The Output Hamming Matrix");
    break;
  case 2 :
    PrintMatrix(all.A_Matrix, all.Exemplars, all.ANodes, all.ColA,
		"The A Exemplar Matrix");
    PrintMatrix(all.B_Matrix, all.Exemplars, all.BNodes, all.ColB,
				 "The Corresponding B Exemplar Matrix");
    PrintMatrix(all.B_TestMatrix, all.Exemplars, all.BNodes, all.ColB,
		"The Input Test Vector Matrix");
    PrintMatrix(all.A_OutTestMatrix, all.Exemplars, all.ANodes, all.ColA,
		"The Output Matrix");
    PrintHamming(all.HamIn, all.Exemplars, all.Exemplars,
				     "The Input Test Hamming Matrix");
    PrintHamming(all.HamOut, all.Exemplars, all.Exemplars,
				     "The Output Hamming Matrix");
    break;
  }

}

/******************************** End PrintResults() *********************/



/******************************** Results() ******************************/
Results(struct start all)

  {
  wipescrn();
  gotoxy(5,5);
  printf("How do you want the results? \n\n");
  gotoxy(5,6);
  printf(" 1. Show the results on screen.\n");
  gotoxy(5,7);
  printf(" 2. Print the results. \n");
  gotoxy(5,8);
  printf(" 3. Show and Print the results. \n");
  gotoxy(5,9);
  printf(" 4. Save the results to a file. \n");
  gotoxy(5,10);
  printf(" Your choice :  ");
  scanf("%d", &choice2);
  switch(choice2)
    {
    case 1 :
       ShowResults(all);
       break;

    case 2 :
       PrintResults(all);
       break;

    case 3 :
       ShowResults(all);
       PrintResults(all);
       break;

    case 4 :
       SaveResults(all);
       break;
     }
  }


/*************************** CalcHamming() ****************************/
CalcHamming(int **inarray1, int **inarray2, int **outarray,
                                       int icntr, int jcntr, int kcntr)
   {
   int i, j, k;
   loopi(icntr)
      loopj(jcntr)
	 {
	 outarray[i][j] = 0;
	 loopk(kcntr)
	    {
	    if ( !(inarray1[i][k] == inarray2[j][k]))
	       outarray[i][j]++;
	    }
	 }
   }
/************************* End CalcHamming() *****************************/

/************************* ShowHamming() *********************************/
ShowHamming( int **inarray, int icntr, int jcntr, int offset, char *title)
   {
   int i, j;
   if( offset == 0)
      {
      gotoxy(20,5);
      printf("%s\n", title);
      loopi(icntr)
      {
	gotoxy(5,6+i);
	printf("\n");
	loopj(jcntr)
	  printf("  HD[%1d,%1d] = %3d",i,j,inarray[i][j]);
      }
      printf("\n");
      }
   else
      {
      gotoxy(20,15);
      printf("%s\n", title);
      loopi(icntr)
      {
	gotoxy(5,17+i);
	printf("\n");
	loopj(jcntr)
	  printf("  HD[%1d,%1d] = %3d",i,j,inarray[i][j]);
      }
      printf("\n");
      }
   }
/***************************** End ShowHamming **************************/

/************************* PrintHamming() *********************************/
PrintHamming( int **inarray, int icntr, int jcntr, char *title)
{
int i, j;
fprintf(stdprn,"\n\n         %s",title);
loopi(icntr)
  {
  fprintf(stdprn,"\n");
    loopj(jcntr)
    fprintf(stdprn,"  HD[%1d,%1d] = %3d",i,j,inarray[i][j]);
  }
  fprintf(stdprn,"\n");
}
/***************************** End PrintHamming() **************************/

/*************************   SaveHamming() *********************************/
SaveHamming( FILE *fout, int **inarray, int icntr, int jcntr, char *title)
{
int i, j;
fprintf(fout,"\n\n         %s",title);
loopi(icntr)
  {
  fprintf(fout,"\n");
    loopj(jcntr)
    fprintf(fout,"  HD[%1d,%1d] = %3d",i,j,inarray[i][j]);
  }
  fprintf(fout,"\n");
}
/***************************** End SaveHamming() **************************/

/**************************** SaveResults() *****************************/
SaveResults(struct start all)
{
FILE *fout;
char outfile[81];
wipescrn();
gotoxy(5,5);
printf("Enter Output File Name:  ");
gotoxy(5,30);
scanf("%s",outfile);
if (( fout = fopen(outfile, "w")) == NULL)
   {
   printf("\n *** I Can't Write to %s ***", outfile);
   exit(1);
   }
fprintf(fout,"Seed = %5d   Bitchange = %2d",seed, bitchange);
switch (choice)
  {
  case 1 :
    SaveMatrix(fout, all.A_Matrix, all.Exemplars, all.ANodes, all.ColA,
		"The A Exemplar Matrix");
    SaveMatrix(fout, all.B_Matrix, all.Exemplars, all.BNodes, all.ColB,
				    "The Corresponding B Exemplar Matrix");
    SaveMatrix(fout, all.A_TestMatrix, all.Exemplars, all.ANodes, all.ColA,
		"The Input Test Vector Matrix");
    SaveMatrix(fout, all.B_OutTestMatrix, all.Exemplars, all.BNodes, all.ColB,
		"The Output Matrix");
    SaveHamming(fout, all.HamIn, all.Exemplars, all.Exemplars,
		 "The Input Test Hamming Matrix");
    SaveHamming(fout, all.HamOut, all.Exemplars, all.Exemplars,
		 "The Output Hamming Matrix");
    break;
  case 2 :
    SaveMatrix(fout, all.A_Matrix, all.Exemplars, all.ANodes, all.ColA,
		"The A Exemplar Matrix");
    SaveMatrix(fout, all.B_Matrix, all.Exemplars, all.BNodes, all.ColB,
				 "The Corresponding B Exemplar Matrix");
    SaveMatrix(fout, all.B_TestMatrix, all.Exemplars, all.BNodes, all.ColB,
		"The Input Test Vector Matrix");
    SaveMatrix(fout, all.A_OutTestMatrix, all.Exemplars, all.ANodes, all.ColA,
		"The Output Matrix");
    SaveHamming(fout, all.HamIn, all.Exemplars, all.Exemplars,
				     "The Input Test Hamming Matrix");
    SaveHamming(fout, all.HamOut, all.Exemplars, all.Exemplars,
				     "The Output Hamming Matrix");
    break;
  }
fclose(fout);
}

/******************************** End SaveResults() *********************/

/*************************** SaveMatrix *********************************/
SaveMatrix(FILE *fout, int **inarray, int icntr, int jcntr, int kcntr, char *title)
   {
   int i, j, k;
   fprintf(fout,"\n                  %s\n\n",title);
   loopi(icntr)
     {
     loopj(jcntr)
	{
	fprintf(fout,"%3d",inarray[i][j]);
	if (((j+1) % kcntr) == 0)
	   fprintf(fout,"\n");
	}
     fprintf(fout,"\n");
     }
   fprintf(fout,"\n");
   }
	      /************  End Of SaveMatrix ****************/




int *IVECTOR(int x,char what[])
{
int *item, i;
item = (int *) calloc(sizeof(int),x);
if(item == NULL) {printf("Alloc %s  ",what);exit(0);}
loopi(x) item[i] = 0;
return item;
}


int freeIMATRIX(int rows, int **m)
   {
   int i;
   for (i=rows-1; i >=0; i--) free((char*) m[i]);
   free((char*) m);
   }

int **IMATRIX(int x, int y, char what[])
{
int i, j, **item;
item = (int **)calloc(sizeof(int *), x);
if(item == NULL){printf("First pointer Dalloc Error in %s\n", *what);exit(0);}
loopi(x){ item[i] = (int*)calloc(sizeof(int),y);
	if(item[i] == NULL) {printf("Dalloc allocation error ");
			     printf("error in %s ", *what);exit(0);}}
loopi(x)loopj(y) item[i][j]= 0;
return item;
}

