
#####################################################################
#
#  BLACKBOX.PL - window manager-specific code for wmtheme
#

@fallback_dirs = (
  '/usr/local/share/Blackbox/styles',
  '/usr/X11R6/share/Blackbox/styles',
  '/usr/share/Blackbox/styles',
  '/usr/share/blackbox/styles');
$wm_executable = 'blackbox';
$downloadsite = 'bb.themes.org';
$configfile = &find_bbdir;

$bbdir = $configfile;
$bbdir =~ s|(.*)/.*|$1|;

$image_bg = '';

# Image viewer options:
%xv = (
  default => '-root -quit',
  scaleopt => '-max',
  tileopt => '',
  centeropt => '-rmode 5' );
%Esetroot = (
  default => '',
  scaleopt => '-scale',
  tileopt => '',
  centeropt => '-center' );
%wmsetbg = (
  default => '',
  scaleopt => '--scale',
  tileopt => '--tile',
  centeropt => '--center' );
%view = (
# I don't have view, and can't find it anywhere, so I don't know
# how to make it scale or center ... email me (auknight@postmark.net) if
# you know the correct syntax ...
  default => '-onroot',
  scaleopt => '',
  tileopt => '',
  centeropt => '' );

####
# You can set this to zero if you want to play around with activation ...
# It works, just a real pain (ie you can't have blackbox running when you
# activate it).  It's usually just easier to select the style from your
# menu.
$unsupported{'activation'} = 1;

if ($> == 0) {
  $bgdir = "backgrounds";
  $styledir = "styles";
  $globalthemedir = "$bbdir/$styledir";
  $localthemedir = "$bbdir/$styledir";
  $globalconfig = $configfile;
} else {
  $bgdir = "Backgrounds";
  $styledir = "Styles";

  foreach (@fallback_dirs) {
    if (-d $_) {
      $globalthemedir = $_;
      dbugout(1,"Found $globalthemedir as global theme dir.") if $debug;
      last;
    }
  } unless ($globalthemedir) {
    print "WARNING: No global dir found.\n";
  } else {
    $globalconfig = "$globalthemedir/../menu" unless ($globalconfig);
  }
  dbugout(1, "Global menu file found as $globalconfig") if $debug;
  
  $localthemedir = "$bbdir/$styledir";
}


# Determine WM version.

if (`blackbox -version` =~ /^\S+\s+(\d+\.\d+)/) {
  $blackbox_version = $1;
  dbugout(1, "Blackbox version recorded as $blackbox_version") if $debug;
} else {
    $blackbox_version = 0;
}

  ###
  #
  #  WM_PROCESSCFG
  #
  ###

sub wm_processcfg {
  my ($configtext, $themepathraw, $themedir, @themedirs, @dirs, $entry, $oid);

#  unless (-r $configfile and -d "$bbdir/$bgdir" and -d "$bbdir/$styledir") {
  unless (-r $configfile) {
    if ($> == 0) {                    
      print "Couldn't find your global config file.  Please enter its";
      print " location\n";
      GLOBAL: {
        print ": ";
        chomp($configfile = <STDIN>);
        unless (-r $configfile) {
          redo GLOBAL;
        }
      }
    
    } else {
      print "You don't seem to have a .blackbox dir and/or Menu file in your home dir.\n";
      print "Enter 'y' to make a user dir now, or enter a different dir,\n";
      print "eg, /home/foo/bb_config/, if you have your bb info there.\n";
      LOCAL: {
        print ": ";
        chomp(my $response = <STDIN>);

        if ($response =~ /^y/i) {
          &makeuser_blackbox("$home/.blackbox");
        } elsif (-r "$response/Menu") {
          $bbdir = $response;
          $configfile = "$response/Menu";
        } elsif ($response =~ /^n/i) {
          &choke("No user dir found.\n");
        } else {
          redo LOCAL;
        }
      }
    }
  }
  unless (-d "$bbdir/$bgdir" and -d "$bbdir/$styledir") {
    dbugout(1,"Either no backgrounds dir or styles dir found: creating.") if $debug;
    &makeuser_blackbox("$bbdir");
  }

#  wm_findthemes($globalthemedir, (stat $globalthemedir)[4]) if $globalthemedir;
#  wm_findthemes($localthemedir, (stat $localthemedir)[4]) if $localthemedir;
  wm_findthemes($configfile);
}

  ###
  #
  #  WM_FINDTHEMES
  #
  ###

sub wm_findthemes {
  my ($file) = shift;
  my ($oid, $entry, $dir, @contents, $name);

  @contents = split(/\n/,slurpfile($file));
#  print "First line of config file: $contents[0]\n";

  dbugout(1,"WM_FINDTHEMES: called with file $file") if $debug;

  foreach (@contents) {
    next if /^#/;
#    print "WM_FINDTHEMES: evaluating line: $_ :::::\n";
#    sleep 1;

    if (/\[style\]\s*\((.*)\).*?\{(.*)\}/) {
      $name = $1;
      $entry = $2;

      unless (-f $entry) {
        print "Found entry '$name' in menu file, but doesn't exist: skipping.\n";
        next;
      }
      dbugout(2,"WM_FINDTHEMES: style file found, adding") if $debug;

      $dir = $entry;
      $dir =~ s|(.*)/.*|$1|;
      $oid = (stat $dir)[4];
      $name =~ s/\\(.)/$1/g;	# Deal with things like 'Steel \(Default\)'

      $avthemes{$name} = [$entry, $oid, (stat $entry)[9]];
    } elsif (/\[include\]\s*\((.*)\)/ && -f $1) {
      dbugout(1,"WM_FINDTHEMES: found include tag: following.");
      wm_findthemes($1);
    } elsif (/\[stylesdir\]\s*\((.*)\)/) {
      findthemes_dir($1, (stat $1)[4]);
    }
  }
}

sub findthemes_dir {
  my ($dir, $oid) = @_;
  my ($entry, $contents, $name);

  opendir THEMEDIR, $dir or &choke("Couldn't opendir $dir: $!");

  dbugout(1,"WM_FINDTHEMES: called with dir $dir") if $debug;

  FILE: foreach $entry (readdir THEMEDIR) {
    next if $entry =~ /^\.\.?$/;
    $entry = "$dir/$entry";

    dbugout(2,"FINDTHEMES_BLACKBOX: evaluating \"$entry\"") if $debug;
#    if (-d $entry) {
#      # fixfix - RECURSION WILL ABORT -- ok, it's just commented out now.
#      &wm_findthemes($entry, $oid);	# Eventually it'll just take themes
#    }					# from the user's menu file ...
    if (-f $entry) {
#    else {
      $contents = &slurpfile($entry);

      if ($contents =~ /[.*]\s*(.*)\s*{.*}/) {
        dbugout(2,"WM_FINDTHEMES: menu file: skipping") if $debug;
        next FILE;
      } elsif ($contents =~ /\w+\.\w+.*:\s*\w*/) {
        dbugout(2,"WM_FINDTHEMES: style file found, adding") if $debug;
        $name = $entry;
        $name =~ s[/.*/][];
        $avthemes{$name} = [$entry, $oid, (stat $entry)[9]];
      } else {
        dbugout(2,"WM_FINDTHEMES: couldn't recognize file $entry") if $debug;
      }
    }

  }
  closedir THEMEDIR or &choke("Couldn't closedir $dir: $!");
}

  ###
  #
  #  WM_QUERYDL
  #
  ###

sub wm_querydl {
  my $themetext = shift;

  return &query_themes_org('bb.themes.org', $themetext);
}

  ###
  #
  #  WM_INSTALLTHEME
  #
  ###

sub wm_installtheme {
  my ($tempdir, $filename, $defname) = @_;
  my ($destthemedir,$file,$themename,$realfile);

###  Determine destination directory

  $destthemedir = "$bbdir/$styledir";

###  Make sure the theme's name doesn't collide with an existing theme
###  perhaps this could be common code; --> (wants improvement)

  @files = filefind($tempdir);

  STYLEFILE: foreach $file (@files) {
    my $type = &findfiletype_blackbox($file);
    if ($type eq "style") {
      $realfile = $file;
      $realfile =~ s|.*/||;
      last STYLEFILE;
    }
  }

  if (!$realfile) {
    &choke("Couldn't find a style file for theme");
  }

  if ($defname) {
    $themename = $defname;
  } else {
    $themename = $realfile;
  }
  $themename = check_name_collision($themename);

  foreach $file (@files) {
    my $type = &findfiletype_blackbox($file);

    if ($type eq "image") {
      $image_bg = $file;
      $image_bg =~ s|.*/(.*?)|$bbdir/$bgdir/$1|;
      &forktick("mv \"$file\" \"$bbdir/$bgdir/\"");
    } elsif ($type eq "style") {
      &forktick("mv \"$file\" \"$destthemedir/\"");
    } elsif ($type eq "not needed") {
      unlink($file);
    } else {
      &choke("Logic error.  Got invalid \"$file\" type from findfiletype_blackbox sub.");
    }
  }
  dbugout(1,"FILES FOUND: @files") if $debug;

  dbugout(2,"INSTALL: adding new theme to user menu.") if $debug;
  &edit_menu_blackbox($themename,$realfile);
  &edit_theme_blackbox($realfile);

#  print "Installation of \"$themename\" complete.\n";
  $avthemes{$themename} = ["$destthemedir/$themename", $>, (stat "$destthemedir/$themename")[9]];
  delete $cleanups{'destination'};
  $themename;
}

  ###
  #
  #  WM_ACTIVATETHEME
  #
  ###

sub wm_activatetheme {
  my $theme = shift;
  my $response;

  # Still warn them though.

  print "In order to activate a theme, blackbox must not be running.\n";
  print "If you're running blackbox now, you may kill it and then continue.\n";
  print "\nWARNING: Theme activation in blackbox has been known to cause\n";
  print "problems.  Use at your own risk!\n\n";
  print "Enter 'y' to continue, or 'n' to skip theme activation.\n";
  print "(You may skip theme activation and this warning message completely\n";
  print "by using the '-I' option for downloading/installing.)\n";

  until ($response) {
    print ": ";

    chomp($response = <STDIN>);
    if ($response =~ /^y/) {
      $response = 1;
    } elsif ($response =~ /^n/) {
      return;
    } else {
      $response = '';
    }
  }

  unless (-f $avthemes{$theme}[0]) {
    &choke("couldn't activate theme: no such file: $avthemes{$theme}[0]");
  }
  my $rcfile = "$home/.blackboxrc";
  if (-f $rcfile) {
    my $rc_contents = &slurpfile($rcfile);
    $rc_contents =~ s|session.styleFile:(.*?)/.*\n?|session.styleFile:$1$avthemes{$theme}[0]\n|;
    &stowfile($rcfile,$rc_contents);
  } else {
    &stowfile($rcfile,"session.styleFile:\t$avthemes{$theme}[0]");
  }  
}

sub edit_menu_blackbox {

# One note: this assumes that either .blackboxrc or the file that
# .blackboxrc points to points to $configfile ... which should be checked
# (especially for users who are installing a user dir for blackbox for the
# first time).

  my $theme = shift;
  my $filename = shift;
  my (@config_contents,$line,$added);

  if (!$theme) {
    &choke("Parse error: No theme name given to edit_menu_blackbox sub.");
  }
  dbugout(1,"MENU: adding $theme to menu file.") if $debug;

  @config_contents = split(/\n/,&slurpfile($configfile));

  foreach (@config_contents) {
    next if /^#/;
#    if (s|(.*)(\[style\].*)|$1\[style\] \($theme\)\t\{$bbdir/$styledir/$theme\}\n$1$2|ix)
    if (m|(.*)\[submenu\]\s*\(Styles\).*|i) {
      $_ .= "\n$1  \[style\] \($theme\)\t\{$bbdir/$styledir/$filename\}";
      $added = 1; 
      last;
    }
  }

  unless ($added) {
    # If there aren't any themes in the menu, add the new one to the end.
    foreach (@config_contents) {
      next if /^#/;
      if (m|\[begin\]|) {
        $_ .= "\n";

        $_ .= "  [submenu] (Styles)\n";
        $_ .= "    [style] ($theme)\t{$bbdir/$styledir/$filename}\n";
        $_ .= "  [end]";

        $added = 1;
      }
    }
  }

  unless ($added) {
    my $old = $config_contents[0];
    $config_contents[0] = "[submenu] (Styles)\n";
    $config_contents[0] .= "  [style] ($theme)\t{$bbdir/$styledir/$filename}\n"; 
    $config_contents[0] .= "[end]\n";
    $config_contents[0] .= "$old" if $old;

  }

  dbugout(1,"Writing to menu file") if $debug;
  &stowfile($configfile,join("\n",@config_contents));

  print "Installed \"$theme\" - Select from root menu to activate.\n";
}

sub edit_theme_blackbox {
  my $theme = shift;
  $theme = "$bbdir/$styledir/$theme";

  my $bgcmd = &find_bgcmd_blackbox;
  my @theme_contents = split(/\n/, &slurpfile($theme));
  my ($image,$viewer,$options,$found,$total_theme);

  THEME_LINE: foreach (@theme_contents) {
    next if /^!/;
    if (m|rootCommand:\s*(.*)|) {
      $found = 1;
      $image = $1;
      $viewer = $1;
      $options = $1;

      dbugout(1,"EDIT_THEME: current line in style file: '$_'") if $debug;

      $viewer =~ s/(.*?)\s.*/$1/;
      $image =~ s|$viewer.*?([~\w\d\.]*/.*)|$1| or $image = "";
      $image =~ s|~/|$home/|;
      $options =~ s|$viewer\s*(.*)\s+[~\w\d\.]*/?.*|$1| or $options = "";
      if ($viewer eq 'bsetroot' and $options !~ /-image/) {
        next THEME_LINE;
      }

      dbugout(1,"EDIT_THEME: image in style file: $image") if $debug;

      unless (-f $image) {
        dbugout(1,"EDIT_THEME: image in style file doesn't exist: changing to $image_bg") if $debug;
        $image = $image_bg;
      }


#      `which $viewer`;
#      unless ($? or $viewer eq 'bsetroot') {

      if (findexe($viewer) and $viewer ne 'bsetroot') {
        dbugout(1,"EDIT_THEME: current rootCommand (using $viewer with '$options') ok ... keeping.") if $debug;
        $_ = "rootCommand:\t$viewer $options $image";
        dbugout(1,"EDIT_THEME: changing line to '$_'") if $debug;
      } else {

        if ($viewer eq 'xv') {
        # xv has some options the other viewers don't, so we have to
        # compromise a bit if the user doesn't have 'xv' installed.
          &parse_xv_options($options);
        }

        # Now this part gets rather ugly ...
        dbugout(1,"EDIT_THEME: bad rootCommand: fixing.") if $debug;
  
        if ($$viewer{scaleopt} and $options =~ $$viewer{scaleopt}) {
          $_ = "rootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{scaleopt} $image";
        } elsif ($$viewer{tileopt} and $options =~ $$viewer{tileopt}) {
          $_ = "rootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{tileopt} $image";
        } elsif ($$viewer{centeropt} and $options =~ $$viewer{centeropt}) {
          $_ = "rootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{centeropt} $image";
      
        } elsif ($$viewer{scaleopt}) {
          if ($$viewer{tileopt}) {
            if ($$viewer{centeropt}) {
              print "Couldn't determine how to display bg image on root window:\n";
              print "  Assuming scale.\n";
              $_ = "rootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{scaleopt} $image";
            } else {
              $_ = "rootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{centeropt} $image";
            }
          } else {
            if ($$viewer{centeropt}) {
              $_ = "rootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{tileopt} $image";
            } else {
              print "Couldn't determine how to display bg image on root window:\n";
              print "  Assuming tile.\n";
              $_ = "rootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{tileopt} $image";
            }
          }

        } elsif ($$viewer{tileopt}) {
          if ($$viewer{centeropt}) {
            $_ = "rootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{scaleopt} $image";
          } else {
            print "Couldn't determine how to display bg image on root window:\n";
            print "  Assuming scale.\n";
            $_ = "rootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{scaleopt} $image";
          }

        } elsif ($$viewer{centeropt}) {
          print "Couldn't determine how to display bg image on root window:\n";
          print "  Assuming scale.\n";
          $_ = "rootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{scaleopt} $image";

        } else {
          print "Couldn't determine how to display bg image on root window:\n";
          print "  Assuming scale.\n";
          $_ = "rootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{scaleopt} $image";
        }
      }
    }
  $total_theme = join("\n",@theme_contents);
  } unless ($found) {
    if ($image_bg) {
    # This should probably guess from the size of the image whether it
    # should be scaled or tiled ...
      print "Found image, but no command to display it:\n";
      print "  Assuming scale.\n";       
      $total_theme .= "\nrootCommand:\t$bgcmd $$bgcmd{default} $$bgcmd{scaleopt} $image_bg";
    }
  }
  dbugout(1,"Rewriting to $theme.") if $debug;
  &stowfile($theme,$total_theme);
}

  ###
  #
  #  WM_UNINSTALLTHEME
  #
  ###

sub wm_uninstalltheme {
#  &choke("theme removal for blackbox has not been written!");

  my $themename = shift;

  dbugout(1,"WM_UNINSTALLTHEME: removing $themename") if $debug;

  my $themecontents = &slurpfile($avthemes{$themename}[0]);
  my $menucontents = &slurpfile("$configfile");

  if ($themecontents =~ m|^\s*rootCommand:.*?(/\S*)\n?|) {
    unlink($1) or &choke("couldn't unlink $1: $!");
  }

  $menucontents =~ s|^\s*\[style\].*\{$avthemes{$themename}[0]\}.*?\n?||m;

  &stowfile("$configfile",$menucontents);
  unlink($avthemes{$themename}[0]);
}

sub makeuser_blackbox {
  my $installdir = shift;

  unless (-d "$installdir") {
    dbugout(1,"making dir $installdir") if $debug;
    mkdir $installdir, 0755 or &choke("Cannot create blackbox user dir: $!");
  }

  unless (-d "$installdir/$bgdir") {
    dbugout(1,"making dir $installdir/$bgdir/") if $debug;
    mkdir "$installdir/$bgdir", 0755 or &choke("Cannot create blackbox Backgrounds dir: $!");
  }

  unless (-d "$installdir/$styledir") {
    dbugout(1,"making dir $installdir/") if $debug;
    mkdir "$installdir/$styledir", 0755 or &choke("Cannot create blackbox Styles dir: $!");
  }

    unless (-f "$configfile") {
    dbugout(1,"copying global menu file to local user dir.") if $debug;
    copyfile("$globalconfig","$configfile");
  }

  &edit_rc_blackbox("menufile");

  print "User directory made.\n";
}

  ###
  #
  #  WM_RENAME
  #
  ###

sub wm_rename {
  my $oldname = shift;
  my $newname = shift;
  my @menufile = split(/\n/,&slurpfile("$configfile"));
  my $styledir;

  foreach (@menufile) {
    next if /^#/;

    if (/\[style\]\s*\(\Q$oldname\E\)/) {
      s/\(\Q$oldname\E\)/\($newname\)/;
      s|\{(.*)/\Q$oldname\E\}|\{$1/$newname\}|;
      $styledir = $1;
    }
  }
  $styledir =~ s/~/$home/;
  &stowfile($configfile,join("\n",@menufile));

  my @stylefile = split(/\n/,&slurpfile("$styledir/$oldname"));
  my($bgdir,$oldbg,$bgsuffix);

  foreach (@stylefile) {
    next if /^#/;

    if (/rootCommand:/) {
      s|(\S*)/(.*)|$1/$newname|;
      $bgdir = $1;
      $oldbg = $2;
      $bgsuffix = $2;
      $bgsuffix =~ s/.*\.//;
      s|\Q$newname\E|$newname.$bgsuffix|;
    }
  }
  $bgdir =~ s/~/$home/;
  &stowfile("$styledir/$oldname",join("\n",@stylefile));

  forktick("mv \'$styledir/$oldname\' \'$styledir/$newname\'");
  forktick("mv \'$bgdir/$oldbg\' \'$bgdir/$newname.$bgsuffix\'");
  
   return "$styledir/$newname";
}

sub wm_versionok {
  my $request = shift;
  my $rqver;

  if ($blackbox_version and $request =~ /(\d+\.\d+)/) {
    $rqver = $1;
    dbugout(2, "VERSIONOK: checking $rqver against Blackbox v$blackbox_version")
      if $debug;
    if ($blackbox_version < 0.6) {
      return $rqver < 0.6;
    } elsif ($blackbox_version < 0.7) {
      return $rqver >= 0.6 and $rqver < 0.7;
    } else {
      ## Make modifications here when later versions come out.
      return 1;
    }
  } else {
    dbugout(2, "VERSIONOK: incomprehensible version \"$request\": returning ok")
      if $debug;
    return 1;
  }
}

sub edit_rc_blackbox {
  my $type = shift;
  my $rcfile = "$home/.blackboxrc";
  if ($type eq "menufile") {
    if (-f $rcfile) {
      my $rc_contents = &slurpfile($rcfile);
      unless ($rc_contents =~ s|session.menuFile:(\s*)[^\s]*|session.menuFile:$1$configfile|)
      {
        $rc_contents .= "\nsession.menuFile:\t$configfile";
      }
      &stowfile($rcfile,$rc_contents);
    }
    else {
      dbugout(1,"no rc file found ... making new") if $debug;
      &stowfile($rcfile,"session.menuFile:\t$configfile");
    }
  } else {
    &choke("Internal error: Invalid type \"$type\" given to edit_rc_blackbox sub!");
  }
  print "Your blackbox rc file has been changed.  To take effect,\n";
  print "blackbox must be restarted (ie from the menu).\n";
}
    
 

sub findfiletype_blackbox {
  my $file = shift;
  my ($type,$contents,$name);

  $name = $file;    
  $name =~ s#.*/##;

  $type = &forktick("file -b $file");
  if ($type =~ /image/i) { 
   return("image");
  } elsif ($name =~ /\.bmp$/) {
    # Apperantly ile on most comps don't detect bmp's right ...
    return("image");
  } elsif ($type =~ /text/) {
    $contents = &slurpfile($file);
    if ($contents =~ /\w+\.\w+.*:\s+\w+/) {
      return("style");
    } elsif ($name =~ /\U$name\E/) {     # Files like README, CHANGES, etc.
      return("not needed");
    } elsif ($name =~ /\.lsm$/) {
      return("not needed");
    } else {
      dbugout(1,"Couldn't determine text file type: $file") if $debug;
      return("not needed");
    }
  } else {
    dbugout(1,"Couldn't determine file type: $file") if $debug;
    return("not needed");
  }
}

sub find_bbdir {
################################
# Woah, just looking over .blackboxrc, and realized this
# sub won't work _at all_.  I spent too much time actually writing
# it to just delete it, so here it is ... if I think of a better
# way of actually doing this, it might be used again. -- D'oh, it's
# menuFile, not styleFile ... this sub works now. :-)
  my (@rc_contents,$menufile,$realmenu);
  dbugout(1,"Finding blackbox dir for user.") if $debug;
  if (-f '/etc/debian_version') {
    dbugout(1,"Debian installed: trying .deb configuration") if $debug;
  # Bit of a hack for debian ... see the .deb file.
    if (-f '/usr/share/blackbox/styles/stylesmenu') {
      unless (-f "$home/.blackbox/stylesmenu") {
        mkdir("$home/.blackbox",0755) unless (-d "$home/.blackbox");
        open(NEWFILE,">$home/.blackbox/stylesmenu");
        close NEWFILE;		# Just to create the file.
        unless (-f "$home/.blackbox/stylesmenu") {
          choke("Made $home/.blackbox/stylesmenu file, but doesn't exist!");
        }
      } $realmenu = "$home/.blackbox/stylesmenu";
      return("$home/.blackbox/stylesmenu");
    } else {
      print "WARNING: If you are using debian, you should've installed wmtheme\n";
      print "with the .deb file.  If you aren't running debian, or you _did_\n";
      print "install the .deb file, feel free to flame <auknight\@postmark.net>\n";
      print "\nWould you like to continue anyway? (NOTE: may cause Undesirable\n";
      print "Effects(tm))\n"; 
      my $answer = <STDIN>;
      unless ($answer =~ /^y/i) {
        choke(".deb file must be used for Debian systems!");
      }
    }
  } 
  if (-f "$home/.blackboxrc") {
    @rc_contents = split(/\n/,&slurpfile("$home/.blackboxrc"));
    foreach (@rc_contents) {
      if (m|session.menuFile:\s*(.*)|) {
        $menufile = $1;
        if (-w $menufile) {
          dbugout(1,"FIND_BBDIR: found $1 as menu file.") if $debug;
#          $menufile =~ s|(.*)/.*|$1|;		# Directory of menu
          $realmenu = $menufile;
          last;
        } elsif (-f $menufile) {
          dbugout(1, "FIND_BBDIR: found $1 as menu file, but not writable.") if $debug;
          $globalconfig = $menufile;
#          $menufile =~ s|(.*)/.*|$1|;		# Directory of menu
          unshift(@fallback_dirs,$menufile);
        } elsif (! $realmenu) {
          dbugout(1,"FIND_BBDIR: found $1 as menu file, but doesn't exist: creating.") if $debug;
          $menufile = $1;
#          if ($menufile =~ m|(.*/).*|) {
#            &makeuser_blackbox($1);
#          }
          $realmenu = $menufile;
          last;
        }
      }
    }
  return $realmenu if $realmenu;
#    dbugout(1,"No menuFile declaration found in .blackboxrc: using default.") if $debug;
  } if ($> == 0) {
    foreach (@fallback_dirs) {
      if (-d $_) {
        $menufile = $_;
        $menufile =~ s|(.*)/.*|$1|;
        opendir(BBDIR,$menufile) or choke("Couldn't open blackbox dir: $!");
        foreach (readdir BBDIR) {
          next if /^..?$/;
          if (-f "$menufile/$_") { 
            dbugout(2,"  Checking file $_") if $debug;
            if (slurpfile("$menufile/$_") =~ /\[.*\]\s*\(.*\)/s) {
              return "$menufile/$_";
            }
          }
        }
        closedir(BBDIR);
        dbugout(1,"FIND_BBDIR: Found styles dir, but no menu file: trying next dir.") if $debug;
      }
    } &choke("Couldn't find global blackbox dir!");
  } else {
    dbugout(1,"No .blackboxrc file found: using default ($home/.blackbox/Menu).") if $debug;
    return "$home/.blackbox/Menu";
  }
}

sub find_bgcmd_blackbox {
  my @possible = qw(wmsetbg xv Esetroot view);
  my @actual;
  dbugout(1,"Looking for following image viewers: " . join(" ",@possible)) if $debug;

  foreach (@possible) {
    #`which $_`;
    #unless ($?) {
    if (findexe($_)) {
      dbugout(1,"Found viewer '$_' installed.") if $debug;
#      push(@actual,$_);
      return $_;
    }
  }
  choke("No suitable image viewer found!") unless $actual[0];
  return @actual;
}

sub parse_xv_options {
  my $options = shift;
  my $rmode_number;

  if ($options =~ /-rmode\s*(\d)/) {
    $rmode_number = $1;
  } else {
  # At the moment, we only really worry about 'rmode's.
    return $options;
  }

  if ($rmode_number <= 4) {
    $rmode_number = 0;		# 0-4 == tile
  } elsif ($rmode_number >= 8) {
    $rmode_number = 0;		# 8 and 9 are also tile
  } else {
    $rmode_number = 5;		# 4, 5, and 6 == centered
  }

  $options =~ s/-rmode\s*\d/-rmode $rmode_number/;
  return $options;
}


1;

