
######################################################################
#
#  ICEWM.PL - IceWM-specific code file for wmtheme
#
#  Documentation:
#
#    http://www.stu.uea.ac.uk/~markj/manual/
#    http://icewmfaq.cjb.net/

$downloadsite = 'icewm.themes.org';
$wm_executable = 'icewm';
my $icewm_version = 0;

  ###
  #
  #  WM_PROCESSCFG
  #
  ###

sub wm_processcfg {
  my (@directories, $dir);

  my @guess_dirs = (
    "$home/.icewm/themes",
    '/usr/local/lib/X11/icewm/themes',   #  DEFAULT
    '/usr/lib/X11/icewm/themes',         #  Debian
    '/usr/X11R6/lib/X11/icewm/themes',   #  Red Hat
    '/usr/local/share/icewm/themes',     #  A guess
    '/usr/X11R6/share/icewm/themes'      #  What Slackware would probably do
    );

  @directories = verify_themedirs(0, @guess_dirs);
  $tempdir_base = $localthemedir if $localthemedir;

  read_dircache();

  if (%cached_dirs) {
    @directories = dircachecheck(@directories);
  }

  foreach $dir (@directories) {
    dbugout(1, "WM_PROCESSCFG: entering directory $dir") if $debug;
    opendir DH, $dir or choke("can't opendir $dir: $!");
    $cached_dirs{$dir} = (stat $dir)[9];

      # Directories coming from the cache may not be the global/local
      # theme dir (a "main" theme dir), and a "main" theme dir must
      # be handled differently from an individual theme's dir; thus
      # this if/else.

    if ($dir eq $localthemedir
        or $dir eq $globalthemedir) {

      while (defined($entry = readdir DH)) {
         ## IceWM won't recognize a theme dir if there's no default.theme

        registerthemedir_icewm("$dir/$entry") if
          $entry !~ /^\.\.?$/ and
          -d "$dir/$entry" and
          -f "$dir/$entry/default.theme";
      }
      closedir DH;

    } elsif (-f "$dir/default.theme") {

       # Since this is an individual theme's dir, one doesn't look into
       # it for individual theme dirs, but rather use it if it's valid
       # ( == has the default.theme file).

      registerthemedir_icewm($dir);

    }
  }
}

sub registerthemedir_icewm {
  my $dir = shift;
  my $oid = (stat $dir)[4];
  $cached_dirs{$dir} = (stat _)[9];
  my ($entry, $name);

  choke("can't grok directory $dir") unless ($name) = $dir =~ /([^\/]+)$/;

   ##   notes
   #
   #  - icewm won't recurse into subdirs
   #  - the name of the directory is the name of the theme
   #  - themes are required to be in a subdirectory (no name.theme
   #    plopped into the themes dir)
   #

  dbugout(2, "REGDIR_ICEWM: scanning directory $dir") if $debug;
  opendir STD, $dir or choke("can't opendir $dir: $!");
  while (defined($entry = readdir STD)) {
    if (!-d "$dir/$entry" and $entry =~ /(.+)\.theme$/) {
      if ($1 eq 'default') {
        dbugout(2, "REGDIR_ICEWM: keeping Default of \"$name\"") if $debug;
        $avthemes{$name} = [$dir, $oid, (stat $dir)[9]];
      } else {
        dbugout(2, "REGDIR_ICEWM: keeping \"$name/$1\"") if $debug;
        $avthemes{"$name/$1"} = ["$dir/$entry", $oid, (stat "$dir/$entry")[9]];
      }
    }
  }
  closedir STD;
}


  ###
  #
  #  WM_QUERYDL
  #
  ###

sub wm_querydl {
  my $themetext = shift;

  return query_themes_org('icewm.themes.org', $themetext);
}

  ###
  #
  #  WM_INSTALLTHEME
  #
  ###

sub wm_installtheme {
  my ($location, $filename, $defname) = @_;

  if (!-d $location) {
    choke("can't use $location as a theme archive");
  }

  my ($themename, $possiblename, $destthemedir, $answer, $cdir);

  if ($> == 0) {
    if (!$globalthemedir) {
      choke("no global theme directory could be identified",
        "perhaps IceWM is not installed on this system?");
    }
    $destthemedir = $globalthemedir;
  } else {
    userinstall_icewm();
    $destthemedir = $localthemedir;
  }

  $cdir = findsubdirwithfile($location, 'default.theme') or
    choke("bad theme - no \"default.theme\" found");

  if ($defname) {
    $possiblename = $defname;
  } elsif ($cdir =~ /\/([^\/]+)$/) {
    $possiblename = $1;
  } else {
    choke("internal error: can't grok $cdir");
  }
  $themename = check_name_collision($possiblename);

  forktick("mv \"$cdir\" \"$destthemedir/$themename\"");
  registerthemedir_icewm("$destthemedir/$themename");

  $themename;
}

  ###
  #
  #  WM_ACTIVATETHEME
  #
  ###

sub wm_activatetheme {
  my ($themename, $explicit) = @_;
  my ($themefile, $prefs);

  if (!$icewm_userinstalled) {
    if ($explicit) {
      userinstall_icewm();
    } else {
      return;
    }
  }

  if (-d $avthemes{$themename}[0]) {
    if ($avthemes{$themename}[0] =~ /\/([^\/]+)$/) {
      $themefile = "$1/default.theme";
    } else {
      choke("internal error: can't grok theme dir $avthemes{$themename}[0]");
    }
  } elsif ($avthemes{$themename}[0] =~ m|/([^/]+/[^/]+)$|) {
    $themefile = $1;
  } else {
    choke("internal error: can't grok theme file $avthemes{$themename}[0]");
  }

  $prefs = slurpfile("$home/.icewm/preferences");
  if ($prefs =~ s/^\s*Theme\s*=.*$/Theme=\"$themefile\"/m) {
  } else {
    $prefs .= "\nTheme=\"$themefile\"\n";
  }
  stowfile("$home/.icewm/preferences", $prefs);
  forktick("killall -HUP $wm_executable");
}

  ###
  #
  #  WM_UNINSTALLTHEME
  #
  ###

sub wm_uninstalltheme {
  my $themename = shift;

  return dirwipe($avthemes{$themename}[0], 1);
}

  ###
  #
  #  WM_RENAME
  #
  ###

sub wm_rename {
  my ($old, $new) = @_;
  my ($newpath, $base, $sub);

  if ($new =~ /\/.*\//) {
    choke("can't handle \"$new\": too many /s");
  } elsif ($new =~ /([^\/]+)\/([^\/]+)/) {
    ($base, $sub) = ($1, $2);
    if ($old !~ /\//) {
      choke("can't rename a base theme to a subtheme");
    } elsif ($old !~ /^\Q$base\E\/[^\/]+\.theme$/) {
      choke("can't shift one subtheme to another base theme");
    }
    $newpath = $avthemes{$old}[0];
    $newpath =~ s/\/\Q$old\E\.theme$/\/$base\/$sub.theme/ or
      choke("internal error: can't find \"$old.theme\" in $avthemes{$old}[0]");
    rename $avthemes{$old}[0], $newpath or
      choke("can't rename \"$avthemes{$old}[0]\" to \"$newpath\": $!");
    return $newpath;
  } else {
    $new =~ s/\///;
    if ($old =~ /([^\/]+)\/([^\/]+)/) {
      ($base, $sub) = ($1, $2);
      $newpath = $avthemes{$old}[0];
      $newpath =~ s/\/\Q$old\E\.theme$/\/$base\/$new.theme/ or
        choke("internal error: can't find \"$old.theme\" in $avthemes{$old}[0]");
      rename $avthemes{$old}[0], $newpath or
        choke("can't rename \"$avthemes{$old}[0]\" to \"$newpath\": $!");
      return $newpath, "$base/$new";
    } else {
      my %subthemes = ();
      foreach (keys %athemes) {
        if (/\/\Q$old\E\/([^\/]+)\.theme$/ and $1 ne 'default') {
          $subthemes{$_} = $1;
        }
      }
      $base = $avthemes{$old}[0];
      $base =~ s/\/\Q$old\E$// or
        choke("internal error: can't find \"$old\" in \"$base\"");
      rename $avthemes{$old}[0], "$base/$new" or
        choke("can't rename \"$avthemes{$old}[0]\" to \"$base/$new\": $!");
      foreach (keys %subthemes) {
        delete $avthemes{$_};
        $avthemes{"$new/$subthemes{$_}"} = ["$base/$new/$_.theme", $>];
      }
      return "$base/$new";
    }
  }
}

  ###
  #
  #  WM_VERSIONOK
  #
  ###

sub wm_versionok {
  my $rqver = shift;
  my $iver;

  dbugout(2, "WM_VERSIONOK: judging version \"$rqver\"") if $debug;
  $rqver =~ /(\d+(\.\d+))/ and $rqver = $1;
  if ($iver = getversion_icewm()) {
    if ($iver < 0.9) {
      return $rqver < 0.9;
    } elsif ($iver < 1.1) {
      return $rqver < 1.1;
    } else {
      return minorvermatch($rqver, $iver);
    }
  } else {
    return 1;
  }
}

sub getversion_icewm {
  if (!defined $icewm_version) {
    my $icever = `icewm -v 2>&1`;
    if ($icever =~ /icewm\s+(\d+\.\d+)/i) {
      $icewm_version = $1;
      dbugout(1, "GETVERSION_ICEWM: dealing with IceWM version $icewm_version")
        if $debug;
    } else {
      dbugout(1, "GETVERSION_ICEWM: got incomprehensible \"$icever\" from icewm -v")
        if $debug;
    }
  }
  $icewm_version;
}

sub userinstall_icewm {
  return if $icewm_userinstalled;

  my ($answer, $preffile);

  if (-d "$home/.icewm" and
      -f "$home/.icewm/preferences" and
      -d "$home/.icewm/themes") {
    $icewm_userinstalled = 1;
    
  } else {

    print "Local IceWM configuration files in ~/.icewm are required. Shall I create this\n";
    print "directory and copy files into it? [Y/n] ";
    $answer = <STDIN>;

    if ($answer =~ /^([yY]|\s*$)/) {

      print "$programname: setting up ~/.icewm\n";

      if (! -d "$home/.icewm") {
        mkdir "$home/.icewm", 0755 or
          choke("can't mkdir $home/.icewm: $!");
      }

      if (! -d "$home/.icewm/themes") {
        mkdir "$home/.icewm/themes", 0755 or
          choke("can't mkdir $home/.icewm/themes: $!");
      }

      if (! -f "$home/.icewm/preferences") {

        if ($globalthemedir and
            $globalthemedir =~ m|(.+)/[^/]+$|
            and -f "$1/preferences") {
          $preffile = "$1/preferences";

        } elsif (-f '/etc/X11/icewm/preferences') {
           ##  - Debian puts preferences here, separate from the global themes dir
          $preffile = '/etc/X11/icewm/preferences';

        } else {
          choke("can't find the system-wide preferences file!");
        }

        copyfile($preffile, "$home/.icewm/preferences");
      }

      $localthemedir = "$home/.icewm/themes";
      $icewm_userinstalled = 1;

    } else {
      choke("cancelled.");
    }
  }
}

1;

