
######################################################################
#
#  sawfish.pl - WM-specific code file for wmtheme
#

$downloadsite = 'sawmill.themes.org';  #  Hasn't changed yet! fixfix when they do
my ($sawfish_version, $client_exe);
getversion_sawfish();

  ###
  #
  #  WM_PROCESSCFG
  #
  ###

sub wm_processcfg {
  my ($themepath, $dir, @dirs, @directories);
  my $smversion = getversion_sawfish();

  if ($themepath = `$client_exe -c '(print theme-load-path)' 2>/dev/null`) {
    dbugout(1, "WM_PROCESSCFG: Sawfish theme path = \"$themepath\"")
      if $debug;
    while ($themepath =~ /[ \(]"(.+?)"(?=[ \)])/g) {
      ($dir = $1) =~ s/^~\//$home\//;
      dbugout(1, "Sawfish theme dir: $dir") if $debug;
      if ($dir =~ /\Q$smversion\E/) {
        unshift @dirs, $dir;
      } else {
        push @dirs, $dir;
      }
    }
  }
  if (!@dirs) {
    @dirs = (
        # Give sawfish precedence over sawmill, in all things
      "$home/.sawfish/themes",
      "/usr/local/share/sawfish/$smversion/themes",   # Default ...
      "/usr/local/share/sawfish/themes",
      "/usr/share/sawfish/$smversion/themes",         # Debian/Red Hat
      "/usr/share/sawfish/themes",

      "$home/.sawmill/themes",
      "/usr/local/share/sawmill/$smversion/themes",   # Default ...
      "/usr/local/share/sawmill/themes",
      "/usr/share/sawmill/$smversion/themes",         # Debian/Red Hat
      "/usr/share/sawmill/themes"
      );
  }

  @directories = verify_themedirs(0, @dirs);
  $tempdir_base = $localthemedir if $localthemedir;
  read_dircache();
  if (%cached_dirs) {
    @directories = dircachecheck(@directories);
  }
  wm_findthemes(@directories);
}

  ###
  #
  #  WM_FINDTHEMES
  #
  ###

sub wm_findthemes {
  my @directories = @_;
  my ($dir, $oid, $entry, $tartest, $name);

  while ($dir = pop @directories) {
    dbugout(1, "WM_FINDTHEMES: scanning $dir") if $debug;
    $oid = (stat $dir)[4];
    $cached_dirs{$dir} = (stat _)[9];
    opendir DH, $dir or choke("can't opendir $dir: $!");
    while (defined($entry = readdir DH)) {
      next if $entry =~ /^\.\.?$/;

      if (-d "$dir/$entry") {
        if (-f "$dir/$entry/theme.jl") {
          dbugout(2, "WM_FINDTHEMES: \"$dir/$entry\" --> $entry")
            if $debug;
          $avthemes{$entry} = ["$dir/$entry", $oid, (stat "$dir/$entry")[9]];
        }
         ## fixfix - what archive types will Sawfish accept?
      } elsif ($entry =~ /(.+)\.(tar\.gz|tgz)$/) {
        dbugout(2, "WM_FINDTHEMES: \"$dir/$entry\" --> $1") if $debug;
        $avthemes{$1} = ["$dir/$entry", $oid, (stat "$dir/$entry")[9]];

      } elsif ($debug) {
        dbugout(2, "WM_FINDTHEMES: \"$dir/$entry\" -- REJECTED (not dir/tarball)");
      }
    }
  }
}


  ###
  #
  #  WM_QUERYDL
  #
  ###

sub wm_querydl {
  my $themetext = shift;

  return query_themes_org($downloadsite, $themetext);
}

  ###
  #
  #  WM_INSTALLTHEME
  #
  ###

sub wm_installtheme {
  my ($location, $possiblename, $defname) = @_;
  my ($themeloc, $themename, $destdir);

## check the $possiblename (archive name), or the subdir name below

###  Determine destination directory

  if ($> == 0) {
    if (!$globalthemedir) {
      choke("no global theme directory could be identified",
        "perhaps Blackbox is not installed on this system?");
    }
    $destdir = $globalthemedir;
  } elsif (!$localthemedir) {
    choke("there is no theme directory for this user. Please run Sawfish first.");
  } else {
    $destdir = $localthemedir;
  }

  $themeloc = findsubdirwithfile($location, 'theme.jl') or
    choke("bad theme - no \"theme.jl\" found");

  if ($defname) {
    $themename = $defname;
  } else {
    if ($themeloc eq $location) {
      if ($possiblename =~ /(.+?)(\.(tgz|tar\.gz|tar\.Z|tar\.bz2|zip|[^\.]+))?$/) {
        $themename = $1;
      } else {
        choke("can't determine theme name from \"$possiblename\"");
      }
    } elsif ($themeloc =~ m|([^/]+)$|) {
      $themename = $1;
    } else {
      choke("can't determine theme name from \"$themeloc\"");
    }
  }
  $themename = check_name_collision($themename);

  forktick("mv \"$themeloc\" \"$destdir/$themename\"");

  $avthemes{$themename} = ["$destdir/$themename", $>,
    (stat "$destdir/$themename")[9]];
  $themename = fixtheme_sawfish($themename, $destdir);
  $themename;
}

  ###
  #
  #  FIXTHEME_SAWFISH
  #
  ###

  #  When a theme is downloaded from themes.org, a name is retrieved from
  #  their database. That takes precedence over the name contained in
  #  the archive, such that the theme's subdirectory will be changed
  #  in wm_installtheme, above.  The name in the (theme-name NAME)
  #  declaration in theme.jl may be still be different, so it must be
  #  changed if the theme is to function...

sub fixtheme_sawfish {
  my ($themename, $destdir) = @_;
  my $themedir = $avthemes{$themename}[0];
  my ($name_set, $internal_name);

  my $themetext = slurpfile("$themedir/theme.jl");

  unless ($themetext =~ /(\(\s*theme-name\s+'(.+?)\s*\))/s
    or $themetext =~ /(\(\s*theme-name\s+\(quote\s+(.+?)\s*\)\s*\))/s) {
    print "warning: theme.jl contains no theme-name\n";
    return;
  }

  $name_set = $1;
  $internal_name = $2;

  if ($internal_name ne $themename) {
    `mv "$destdir/$themename" "$destdir/$internal_name"`;

    delete $avthemes{$themename};

    $avthemes{$internal_name} = [ "$destdir/$internal_name", $>,
      (stat "$destdir/$internal_name")[9]];
    $themename = $internal_name;
  }

  $themename;
}

  ###
  #
  #  WM_ACTIVATETHEME
  #
  ###

sub wm_activatetheme {
  my $themename = shift;

  open (SC, "|$client_exe -q -") or
    choke("cannot execute $client_exe: $!");

    # There seem to be two choices for quoting the theme name here:
    #   '$themename
    #   (quote $themename)
    # Best to use the latter.

  print SC "(setq default-frame-style (quote $themename))\n";
  print SC "(reframe-all-windows)\n";
  close SC;
}

  ###
  #
  #  WM_UNINSTALLTHEME
  #
  ###

sub wm_uninstalltheme {
  my $themename = shift;

  return dirwipe($avthemes{$themename}[0], 1);
}

  ###
  #
  #  WM_RENAME
  #
  ###

sub wm_rename {
  my ($old, $new) = @_;

  my $newpath = $avthemes{$old}[0];
  $newpath =~ s/\/\Q$old\E([^\/]*)$/\/$new$1/ or
    choke("internal error: can't find \"/$old\[...]\" in \"$newpath\"");
  rename $avthemes{$old}[0], $newpath or
    choke("can't rename \"$avthemes{$old}[0]\" to \"$newpath\": $!");
  $newpath;
}

  ###
  #
  #  WM_VERSIONOK
  #
  ###

sub wm_versionok {
  my $request = shift;
  my $smver = getversion_sawfish();
  $smver =~ s/^\D*(\d+\.\d+).*$/$1/;

  if ($smver and $request =~ /(\d+(\.\d*)?)/) {
    dbugout(2, "WM_VERSIONOK: judging version \"$request\"") if $debug;
    $rqver = $1;
    if ($smver < 0.22) {
      return $rqver < 0.22;
    } elsif ($smver < 0.27) {
      return $rqver > 0.18 and $rqver < 0.27;
    } else {
      return minorvermatch($request, $smver);
    }
  } else {
    dbugout(2, "WM_VERSIONOK: can't figure out version request \"$request\"")
      if $debug;
    return 1;
  }
}

sub getversion_sawfish {
  if (!defined $sawfish_version) {
    my $response;

    $sawfish_version = '';

    if ($response = `sawfish --version`) {
      $client_exe = 'sawfish-client';
      $wm_executable = 'sawfish';
    } elsif ($response = `sawmill --version`) {
      $client_exe = 'sawmill-client';
      $wm_executable = 'sawmill';
    } else {
      choke("can't figure out Sawfish's version");
    }

    if ($response =~ /version\s+(\d\S+)/) {
      $sawfish_version= $1;
      dbugout(1, "WM_PROCESSCFG: dealing with Sawfish version $sawfish_version")
        if $debug;
    } else {
      choke("can't figure out Sawfish's version");
    }
  }

  $sawfish_version;
}

1;

