#  Download subs for wmtheme

use Socket;
my $ftp_command;

sub query_themes_org {
  my ($downloadsite, $themetext) = @_;
  my ($discard, $xtext, $url, %themes, @choices, $yourchoice, $minwidth, $i,
    @dispchoices, $size);

  print "$programname: querying for $themetext...";

  $xtext = getrdfquery($downloadsite, 'themetxt', $themetext);
  %themes = rdfparse($xtext, 'name', $themetext);
  if (!(@choices = sort keys %themes)) {
    print "\n";
    choke("query returned no matches for $themetext");
  }

  if ($#choices == 0) {
    $yourchoice = $choices[0];
    print " found \"$yourchoice\"\n";
    if (!$themes{$yourchoice}{'versionok'}) {
      print "This theme was made for $wm_name $themes{$yourchoice}{'version'}, and may not\n";
      print "work properly with the installed version.  Do you want to proceed? [y/N] ";
      $answer = <STDIN>;
      if ($answer !~ /^[yY]/) {
        finish("download cancelled.");
      }
    }
  } else {
    print "\n";
    
    ###  build choices display strings  ###

    $minwidth = getmaxlen(@choices);
    for ($i = 0; $i < @choices; ++$i) {
      $size = bytesdisplay($themes{$choices[$i]}{'ver_size'});
      $dispchoices[$i] = $choices[$i] . ' ' x ($minwidth -
        length($choices[$i]) + 3) . $size . ' ' x (12 - length($size)) .
        localtime($themes{$choices[$i]}{'ver_updated'});
      if (!$themes{$choices[$i]}{'versionok'}) {
          $dispchoices[$i] .= ' ' x (17 - length($size)) .
            "[* $themes{$choices[$i]}{'version'}]";
      }
    }

    print $#choices + 1,
      " themes were returned. Please choose one ([*] = possible incompatibility):\n\n";
    my $choice = getuserchoice(@dispchoices);
    finish("theme retrieval aborted") if $choice < 0;
    $yourchoice = $choices[$choice];
  }

  return $themes{$yourchoice};
}

sub getrdfquery {
  my ($downloadsite, %queryterms) = @_;
  my ($texturl, $url, $discard, $response);

  $texturl = "http://$downloadsite/themes.rdf.phtml?" .
    makecgiline(%queryterms);

  dbugout(2, "GETRDFQUERY: requesting $texturl") if $debug;

  $url = parse_url($texturl);
  ($discard, $response) = downloadfile($url);
  $response;
}

sub rdfparse {
  my ($xtext, %searchterms) = @_;
  my ($itemtext, $themename, %themes, $keepver, $verblock,
    $vers, $term, $matches);

  while ($xtext =~ /<item(\s+[^>]*)?>(.+?)<\/item>/gs) {
    $itemtext = $2;
    $matches = 1;
    foreach $term (keys %searchterms) {
      if (entity_text($itemtext, $term) !~ /\Q$searchterms{$term}\E/i) {
        $matches = 0;
      }
    }
    next unless $matches;

    $themename = entity_text($itemtext, 'name');
    $themes{$themename}{'name'} = $themename;
    $themes{$themename}{'origname'} = $themename;
    $themes{$themename}{'id'} = entity_text($itemtext, 'id');
    $themes{$themename}{'author'} = entity_text($itemtext, 'author');
    $themes{$themename}{'category'} = entity_text($itemtext, 'category');
    $vers = entity_text($itemtext, 'vers');

      ## fixfix - the latest good version is found here, but it depends
      ##          on the response containing versions in ascending order,
      ##          which they are at the moment, but it's not a great
      ##          assumption.  I don't just want to 'sort' them either

    $keepver = '';
    foreach (split(/,/, $vers)) {
      if (version_ok($_) or !$keepver or !version_ok($keepver)) {
        $keepver = $_
      }
    }

    if ($keepver) {
      $themes{$themename}{'version'} = $keepver;
      $verblock = entity_contents($itemtext, "ver $keepver");
      foreach (qw(link updated size)) {
        $themes{$themename}{"ver_$_"} = entity_text($verblock, $_);
      }
      $themes{$themename}{"ver_link"} .= "&query=download";
      $themes{$themename}{'versionok'} = version_ok($keepver);
      dbugout(2, "RDFPARSE: recorded \"$themename\" for version \"$keepver\"")
        if $debug;
      delete $themes{$themename} unless $themes{$themename}{'ver_size'} > 0;
    } else {
      delete $themes{$themename};
    }
  }
  %themes;
}

sub entity_text {
  my ($text, $name) = @_;

  my $contents = entity_contents($text, $name);
  $contents =~ s/<\s*(\S*)\b[^>]*>[^<]*<\/\s*\1\b[^>]*>//gis;
  $contents =~ s/&amp;/&/gs;
  $contents;
}

sub entity_contents {
  my ($text, $name) = @_;

  if ($text =~ /<\s*\Q$name\E(\s+[^>]*)?>(.+?)<\/\s*\Q$name\E(\s+[^>]*)?>/si) {
    return $2;
  } else {
    return '';
  }
}

 ########################################################################
 #
 #  DOWNLOADFILE
 #
 #  Grabs a file.  Begins with an HTTP url whose bits are in $dlinfo.
 #
 #  Will follow redirection, use FTP if so redirected, and show a
 #  progress bar if the size of the downloaded file is available.
 #
 #  Has three output modes, depending on the nature of the request
 #  and the filename that's determined when the desired data is
 #  located:
 #
 #   1) No $destination was specified - 
 #
 #      * The file will be placed in a buffer and returned in
 #        a big wad.
 #
 #
 #   2) A $destination was specified, - AND -
 #        -  $extract was false
 #           - OR -
 #        -  $extract was true, but no extraction method could be
 #           determined from the final filename -
 #
 #      * The file will be written into the $destination directory,
 #        with the internally determined filename.
 #
 #
 #   3) A $destination was specified, - AND -
 #      $extract was true             - AND -
 #      An extraction method was determined from the final filename
 #
 #      * The data will be fed through a pipe, that extracts
 #        and writes files into the $destination directory.
 #
 #  INPUTS:
 #
 #    $dlinfo       - a reference to a hash with URL parts
 #    $destination  - optional, a directory in which to save the data
 #    $extract      - boolean, whether to try to extract the data if
 #                    its archive method can be determined
 #    $showprogress - boolean, whether to show a download gauge
 #
 #  OUTPUTS:
 #
 #    A list with two elements:
 #      - The filename
 #      - The data (will be empty if $destination was specified)
 #

sub downloadfile {
  my ($dlinfo, $destination, $extract, $showprogress) = @_;
  my ($socket, $header, $loc);
  
  dbugout(1, "DOWNLOADFILE: getting ", urlstring($dlinfo)) if $debug;

  if ($$dlinfo{'proto'} eq 'ftp') {
    $loc = $dlinfo;
  } else {
    ($socket, $header, $loc) = http_follow($dlinfo);
  }

  my (
    $input,            #  Input filehandle - returned socket or pipe from FTP utility
    $size,             #  Number of bytes to retrieve, if known (for gauge)
    $gauge,            #  String, to be displayed gradually during main transfer
    $gauge_index,      #  Position in gauge
    $buffer,           #  Destination for data if no $destination is passed
    $block,            #  a block of data during the transfer
    $blocksize,        #  amount of data to transfer so that the gauge comes out nice
    $extractcmd,       #  command to pipe data through if $extract is true
    $filename,         #  file to write data to if $extract isn't true
    $caught_sigpipe,   #  flag that becomes true when $extractcmd (above) has an error
    $use_guage,        #  whether the size and gauge were ok, this is used during transfer
    $cols              #  screen columns (width): influences $gauge preparation
    );

  #####
  ##
  ##  SIZE, FILENAME, and INPUT depend on protocol
  ##

  if ($$loc{'proto'} eq 'http') {

      ## -  SIZE  - sometimes available in http  - ##

    if ($header =~ /^content-length:\s+(\d+)/im) {
      $size = $1;
    } else {
      $size = 0;
    }

      ## -  FILENAME  - from url or header in http  - ##

    if ($destination) {
      if ($header =~ /^content-disposition:.*filename=\s*(.+?)\s/is) {
        $filename = $1;
      } elsif ($$loc{'location'} =~ /\/([^\/]+)$/) {
        $filename = cgidecode($1);
      } else {
        abort_bad_data("can't figure out a filename");
      }
    }

      ## -  INPUT  - we get to use this pre-opened socket  - ##

    $input = $socket;

  } elsif ($$loc{'proto'} eq 'ftp') {

      ## -  SIZE  - not available in ftp  - ##

    $size = 0;

      ## -  FILENAME  - from url in ftp  - ##

    if ($destination) {
      if ($$loc{'location'} =~ /([^\/]+)$/) {
        $filename = $1;
      } else {
        abort_bad_data("can't get filename from ftp url!");
      }
    }

      ## -  INPUT  - must pipe from an ftp command  - ##

    close $socket;
    $input = makeftpinput($loc);

  } else {
    abort_bad_data("unhandled protocol \"$$loc{'proto'}\"");
  }


  ##
  ##  (end of protocol-dependent business)
  ##
  #####

 ## --- prepare output filehandle (critical if requested) --- ##

  if ($extract and ($extractcmd =
      get_extractcmd($filename, $destination, 1))) {
    dbugout(1, "DOWNLOADFILE: piping through $extractcmd") if $debug;
    open (DEST, "|$extractcmd") or
      choke("can't pipe through \"$extractcmd\": $!");
  } elsif ($destination) {
    dbugout(1, "DOWNLOADFILE: writing to $destination/$filename") if $debug;
    open (DEST, ">$destination/$filename") or
      choke("can't write $destination/$filename: $!");
  }
  $caught_sigpipe = 0;
  local $SIG{PIPE} = sub { $caught_sigpipe = 1 };


 ## --- prepare progress gauge (no failure mode) --- ##

  $use_gauge = 0;
  if ($showprogress) {
    if ($size) {
      $cols = (screensize())[1];
      $gaugehdr = "($size bytes) [";
      $blocksize = int($size / ($cols - 1 - length($gaugehdr)));
      $blocksize++ if $size / $blocksize > $cols - 2 - length($gaugehdr);
      $gauge = makedlgauge(int($size / $blocksize));
      $use_gauge = 1;
      $gauge_index = 0;
      print $gaugehdr;
    } else {
      print "(length unknown) [";
      $blocksize = 8192;
    }
  } else {
    $blocksize = 8192;
  }


 ## --- transfer data --- ##

  $buffer = '';
  while (read $input, $block, $blocksize) {
    $debugdata .= $block if $debug;
    if ($destination) {
      print DEST $block;
      if ($caught_sigpipe) {
        choke("invalid data received; caught SIGPIPE");
      }
      if ($showprogress) {
        $use_gauge ? print substr($gauge, $gauge_index++, 1) : print '.';
      }
    } else {
      $buffer .= $block;
    }
  }
  print "]\n" if $showprogress;


 ## --- clean up & return --- ##

  modtree($destination) if $extract;
  $debugdata .= "\n[$programname] ***  TRANSFER COMPLETE  ***\n"
    if $debug;
  close $input;
  if ($destination) {
    close DEST or choke("error saving download: $!");
  }
  ($filename, $buffer);
}


  #########################################################################
  #
  #  HTTP_FOLLOW
  #
  #  Receives a reference to an http url hash.
  #
  #  Makes requests, starting with the passed url, following any HTTP
  #  redirections.
  #
  #  Possible outcomes:
  #
  #    *  The server returns 200:  The socket, header, and URL that
  #       led to this are returned.  The data, if any, is ready to be
  #       read from the socket.
  #
  #    *  The server redirects to a protocol other than HTTP: The
  #       header and URL found are returned (undef is returned in
  #       place of the socket).
  #
  #    *  The server returns an error:  Program aborts.
  #
  
sub http_follow {
  my ($dlinfo, $redirections) = @_;

  dbugout(1, "HTTP_FOLLOW: getting ", urlstring($dlinfo)) if $debug;

  my ($httpsock, $header) = http_start($dlinfo);

  if ($header =~ /^http\/\S+\s+(\d+)\s+(.+?)\s*$/im) {
    $httpcode = $1;
    $httpresponse = $2;
  } else {
    close $httpsock;
    abort_bad_data("can't identify HTTP response");
  }

  if ($httpcode eq '200') {
    return ($httpsock, $header, $dlinfo);
  } elsif ($httpcode =~ /^30[12357]$/) {
    $redirections ||= 0;
    if ($redirections > 9) {
      abort_bad_data(
        "server issued $redirections HTTP redirections; quitting");
    } elsif ($header =~ /^location:\s+(\S+)/im) {
      my $newurl = parse_url($1);
      if ($$newurl{'proto'} eq 'http') {
        return http_follow($newurl, ++$redirections);
      } else {
        return (undef, $header, $newurl);
      }
    } else {
      abort_bad_data(
        "server replied \"$httpcode $httpresponse\" but no Location: can be found");
    }
  } else {
    choke("server replied \"$httpcode $httpresponse\"");
  }
}


  #########################################################################
  #
  #  HTTP_START
  #
  #  Initiates an HTTP request.
  #
  #  Receives a reference to a url hash.
  #
  #  Creates a socket and opens a connection to the server; gets the
  #  returned header.  Returns the socket and the header.  Receiving
  #  any remaining data on the socket, if any, is the responsibility of
  #  the caller.
  #

sub http_start {
  my $dlinfo = shift;

  my ($iaddr, $paddr, $header, $line, $request);

  dbugout(1, "HTTP_START: getting ", urlstring($dlinfo)) if $debug;

  $iaddr = gethostbyname($$dlinfo{'server'}) or
    choke("can't resolve \"$$dlinfo{'server'}\": $!");

  $paddr = sockaddr_in($$dlinfo{'port'}, $iaddr) or
    choke("sockaddr_in failure with " . inet_ntoa($paddr) .
      ":$$dlinfo{'port'}: $!");

  socket(HTTPSOCK, PF_INET, SOCK_STREAM, getprotobyname('tcp')) or
    choke("can't get a socket: $!");

  connect(HTTPSOCK, $paddr) or
    choke("can't connect to $$dlinfo{'server'}:$$dlinfo{'port'}: $!");

  select((select(HTTPSOCK), $| = 1)[0]);

# User-Agent: Mozilla/4.7 [en] (X11; I; Linux 2.2.13 i686)
# Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, */*

  $request = "GET $$dlinfo{'location'} HTTP/1.0
User-Agent: wmtheme/$version ($uname_srm)
Host: $$dlinfo{'server'}
Accept-Encoding: gzip
Accept-Language: en
Accept-Charset: iso-8859-1,*,utf-8\n\n";

  $debugdata .= " ============  REQUEST  ============\n" .
    $request .  " -----------------------------------\n" if $debug;

  print HTTPSOCK $request;

  $header = '';
  while ($line = <HTTPSOCK>) {
    $header .= $line;
    $debugdata .= $line if $debug;
    last if $line =~ /^[\r\s]*$/;
  }

  dbugout(2, " ++++++++++++  HEADER  ++++++++++++",
    "$header ----------------------------------") if $debug;

  (\*HTTPSOCK, $header);
}


  #########################################################################
  #
  #  PARSE_URL
  #

sub parse_url {
  my ($url, $noncritical) = @_;
  my %dlinfo;

  dbugout(2, "PARSE_URL: Examining $url") if $debug;
  if ($url =~ m|^(\w+)://([^/]+?)(:(\d+))?(/.*)|) {
    $dlinfo{'proto'} = lc $1;
    $dlinfo{'server'} = $2;
    if ($4) {
      $dlinfo{'port'} = $4;
    } else {
      $dlinfo{'port'} = getservbyname($dlinfo{'proto'}, 'tcp');
    }
    $dlinfo{'location'} = $5 || '/';
    dbugout(2, "PARSE_URL: Result: " . urlstring(\%dlinfo)) if $debug;
  } else {
    choke("can't parse url $url") unless $noncritical;
    dbugout(2, "PARSE_URL: Can't parse.") if $debug;
  }

  \%dlinfo;
}


  #########################################################################
  #
  #  URLSTRING
  #

sub urlstring {
  my $dlinfo = shift;

  "[$$dlinfo{'proto'}] :// [$$dlinfo{'server'}] : [$$dlinfo{'port'}] [$$dlinfo{'location'}]";
}


  #########################################################################
  #
  #  COMBINEURL
  #

sub combineurl {
  my ($url, $link) = @_;

  dbugout(2, "COMBINEURL: doing [", urlstring($url), "]  PLUS  [$link]")
    if $debug;

  my $newurl = parse_url($link, 1);

  if (defined $$newurl{'proto'}) {
    return $newurl;
  } elsif ($link =~ /^\/.+/) {
    $$url{"location"} = $link;
  } elsif ($$url{'location'} =~ /.+\/$/) {
    $$url{'location'} .= $link;
  } else {
    $$url{'location'} =~ s|/[^/]+$|/$link|;
  }
  dbugout(2, "COMBINEURL: Result: ", urlstring($url)) if $debug;

  $url;
}

  #########################################################################
  #
  #  MAKEFTPINPUT
  #

sub makeftpinput {
  my $loc = shift;
  my $ftpsource;

  if (!$ftp_command) {
    my $tmpcmd;
    my @ftp_list = qw(wget curl ncftp);  # This defines the order of preference.
    my %ftpcommands = (
      'wget',    '-O - ftp://HOSTLOCATION',
      'curl',    'ftp://HOST/LOCATION',
      'ncftp',   '-a -c ftp://HOSTLOCATION'
      );
    foreach $tmpcmd(@ftp_list) {
      if (findexe($tmpcmd)) {
        $ftp_command = "$tmpcmd $ftpcommands{$tmpcmd}";
        dbugout(1, "identified FTP command: $ftp_command") if $debug;
        last;
      }
    }
    if (!$ftp_command) {
      choke("no FTP command from [", join(', ', @ftp_list), "] is available");
    }
  }

  $ftpsource = $ftp_command;
  $ftpsource =~ s/HOST/$$loc{'server'}/ or
    choke("internal error: \"$ftpsource\" didn't contain 'HOST'");
  $ftpsource =~ s/LOCATION/$$loc{'location'}/ or
    choke("internal error: \"$ftpsource\" didn't contain 'LOCATION'");
  open FTPIN, "$ftpsource|" or
    choke("can't pipe from $ftpsource: $!");
  return \*FTPIN;
}


  #########################################################################
  #
  #  MAKEDLGAUGE
  #

sub makedlgauge {
  my $glength = shift;

  my @pctmarkers = qw(0 10 20 30 40 50 60 70 80 90 100);
  my $pctlength = length(join('', @pctmarkers));

  if ($glength < 1) {
    return '.';
  } elsif ($glength < $pctlength +10) {
    return '.' x $glength;
  }

  my $i;
  my $smallsect = int(($glength - $pctlength)/ 10);
  my $gauge = $pctmarkers[0];

  $addmarks = substr('385027916', 0, $glength - $pctlength - $smallsect * 10);
  for ($i = 0; $i < 10;) {
    $gauge .= '.' if $addmarks =~ /$i/;
    $gauge .= '.' x $smallsect . $pctmarkers[++$i];
  }

  $gauge;
}

1;

