
######################################################################
#
#  WMAKER.PL - window maker-specific code for wmtheme
#

$downloadsite = 'wm.themes.org';

if (findexe('WindowMaker')) {
  $wm_executable = 'WindowMaker';
} elsif (findexe('wmaker')) {
  $wm_executable = 'wmaker';
} else {
  choke("sorry, can't find \"wmaker\" or \"WindowMaker\"");
}

my $wmaker_version;

  ###
  #
  #  WM_PROCESSCFG
  #
  ##

sub wm_processcfg {
  my (@directories, $dir, $oid, $entry);

  my @guess_dirs = (
    '/usr/local/share/WindowMaker/Themes',     # DEFAULT
    '/usr/share/WindowMaker/Themes',           # Debian/Red Hat
    '/usr/X11R6/share/WindowMaker/Themes'      # Slackware
    );

  my $wmaker_root = $ENV{'GNUSTEP_USER_ROOT'} || "$home/GNUstep";

  if (-d $wmaker_root) {
    push @guess_dirs, "$wmaker_root/Library/WindowMaker/Themes";
  }

  dbugout(2, "WM_PROCESSCFG: Possible directories:\n  [" .
    join("]\n  [", @guess_dirs) . ']') if $debug;

  @directories = verify_themedirs(0, @guess_dirs);
  $tempdir_base = $localthemedir if $localthemedir;

  read_dircache();
  if (%cached_dirs) {
    @directories = dircachecheck(@directories);
  }

  while ($dir = pop @directories) {
    dbugout(1, "WM_PROCESSCFG: scanning $dir") if $debug;
    $oid = (stat $dir)[4];
    $cached_dirs{$dir} = (stat _)[9];
    opendir DH, $dir or choke("can't opendir $dir: $!");
    while (defined($entry = readdir DH)) {
      next if $entry =~ /^\.\.?$/;

      if (-d "$dir/$entry") {
        if ($entry =~ /(.+)\.themed$/) {
          dbugout(2, "WM_PROCESSCFG: \"$entry\" --> theme \"$1\"") if $debug;
          $avthemes{$1} = ["$dir/$entry", $oid, (stat "$dir/$entry")[9]];
        } else {
          dbugout(2, "WM_PROCESSCFG: \"$entry\" ==>> recursing into dir")
            if $debug;
          push @directories, "$dir/$entry";
        }
      } else {
        if ($entry =~ /(.+)\.style$/) {
          dbugout(2, "WM_PROCESSCFG: \"$entry\" --> theme \"$1\"") if $debug;
          $avthemes{$1} = ["$dir/$entry", $oid, (stat "$dir/$entry")[9]];
        } else {
          dbugout(2, "WM_PROCESSCFG: \"$entry\" --> theme \"$entry\"")
            if $debug;
          $avthemes{$entry} = ["$dir/$entry", $oid, (stat "$dir/$entry")[9]];
        }
      }
    }
    closedir DH or choke("can't closedir $dir: $!\n");
  }
}    

  ###
  #
  #  WM_INSTALLTHEME
  #
  ###

sub wm_installtheme {
  my ($location, $filename, $defname) = @_;
  my ($themename, $destthemedir, $stylefile, $i, @files, $mvfiles);

###  Determine destination directory

  if ($> == 0) {
    if (!$globalthemedir) {
       choke("no global theme directory could be identified",
         "perhaps Window Maker is not installed on this system?");
    }
    $destthemedir = $globalthemedir;
  } elsif (!$localthemedir) {
    choke("there is no theme directory for this user; do you need to run wmaker.inst?");
  } else {
    $destthemedir = $localthemedir;
  }

###  Locate style file

  if (-d $location) {
    @files = filefind($location);
    $stylefile = teststyle_wmaker(@files) or
      choke("can't identify a style file; giving up");
    dbugout(1, "STYLE: $stylefile") if $debug;

    # Remove style file from list of files

    for ($i = 0; $i <= $#files; ++$i) {
      if ($files[$i] eq $stylefile) {
        splice(@files, $i, 1);
        last;
      }
    }
  } elsif ($stylefile = teststyle_wmaker($location)) {
    dbugout(1, "STYLE: $stylefile") if $debug;
  } else {
    choke("$location does not appear to be a style file.");
  }

  fixstyle_wmaker($stylefile);

###  Determine the theme's name

  if ($defname) {
    $themename = $defname;
  } else {
    if ($stylefile =~ /\/([^\/]+?)\.themed$/) {
      # a somewhat broken theme does this.. okay, we'll handle it
      $themename = $1;
    } elsif ($stylefile =~ /\/([^\/]+)\.themed\/style$/) {
      $themename = $1;
    } elsif ($stylefile =~ /\/([^\/]+)\/style$/) {
      $themename = $1;
    } elsif ($stylefile =~ /\/([^\/]+?)(\.style)?$/) {
      $themename = $1;
    } elsif ($stylefile =~ /\/([^\/]+)$/) {
      $themename = $1;
    } else {
      $themename = $stylefile;
    }
  }
  $themename = check_name_collision($themename);

###  Create destination directory and move files into it

  $destthemedir .= "/$themename.themed";
  dbugout(2, "WM_INSTALLTHEME: creating destination directory $destthemedir")
    if $debug;

  mkdir $destthemedir, 0755 or
    choke("can't create destination directory $destthemedir: $!");
 
  $cleanups{'wmaker_install'} = $destthemedir;
  forktick("mv \"$stylefile\" \"$destthemedir/style\"");

   ## If $location was a single file, it had to be a style or
   ## we wouldn't have gotten this far, and the previous line
   ## would have moved it already.

  if (-d $location) {
    @files = purge_duplicates_wmaker(@files);
    $mvfiles = join('" "', @files);
    forktick("mv \"$mvfiles\" \"$destthemedir\"");
  }

  $avthemes{$themename} = [$destthemedir, $>, (stat $destthemedir)[9]];
  delete $cleanups{'wmaker_install'};
  $themename;
}

  #######################################################################
  #
  #  PURGE_DUPLICATES_WMAKER
  #
  #  Removes duplicate files from a window maker theme archive,
  #  according to some wmaker-specific rules
  #
  #  This sub supports wm_installtheme
  #

sub purge_duplicates_wmaker {
  my @files = @_;
  my (%files, $filename);

  foreach (@files) {
      ###  Decide whether to keep or discard each file

    if (/\/([^\/]+$)/) {
      $filename = $1;
      if ($filename eq 'style') {
        ###  Always discard extra style files

      } elsif (!exists $files{$filename}) {
        ###  Always keep when there's no name collision
        $files{$filename} = $_;

      } elsif (/\.xvpics\//) {
        ###  Discard same-named ones in .xvpics subdirs
        ###  since they will be thumbnails and mess up the theme
        ###  if we keep them

      } elsif ($files{$filename} =~ /\.xvpics/) {
        ###  Same thing, but the earlier one was in .xvpics
        ###  so let this one replace it

        $files{$filename} = $_;
      } elsif (/\/Themes\/$filename$/) {
        ###  Keep the newer one if it's in Themes subdir

        $files{$filename} = $_;
      }
    }
  }
  values %files;
}

  #######################################################################
  #
  #  FIXSTYLE_WMAKER
  #
  #  Remove directories from paths specified in a window maker style
  #  file. Necessary since all files will be put in their own themed.
  #
  #  This sub supports wm_installtheme
  #

sub fixstyle_wmaker {
  my $stylefile = shift;
  my $styletext = slurpfile($stylefile);
  $pixmapsettings = join ('|', 
    'workspaceback',
    'ftitleback',
    'ptitleback',
    'utitleback',
    'resizebarback',
    'menutitleback',
    'menutextback',
    'iconback');

  $styletext =~ s/\b(($pixmapsettings)\s*=\s*\(\s*.pixmap\s*,\s*)[^",]*\//$1/sgi;
  $styletext =~ s/\b(($pixmapsettings)\s*=\s*\(\s*.pixmap\s*,\s*")[^"]*\//$1/sgi;

  # Remove comments; setstyle doesn't like them
  $styletext =~ s/^\s*#.*$//mg;

  stowfile($stylefile, $styletext);
}

  #######################################################################
  #
  #  TESTSTYLE_WMAKER
  #
  #  Examine some files and return the most likely style file.
  #
  #  This sub supports wm_installtheme
  #

sub teststyle_wmaker {
  my @candidates = @_;
  my (@nonimages, @filetypes, %possibles, $besttype, $filetype, $filename,
    $text, $best);

  foreach (@candidates) {
    push @nonimages, $_ unless
      /\.(gif|jpg|jpeg|png|tif|tiff|xpm|lsm)$/i;
  }
  if (!@nonimages) {
    dbugout(1, "TESTSTYLE: FAILED (1)") if $debug;
    return;
  }

  $besttype = '';
  @filetypes = split("\n", forktick('file -b "' . join('" "', @nonimages) . '"'));
  foreach $filename (@nonimages) {
    $filetype = shift @filetypes;
    dbugout(2, "TESTSTYLE: [ $filetype ] <> [ $filename ]") if $debug;
    if ($filetype eq 'ASCII text') {
      return $filename if $filename =~ /\/style$/;
      if ($besttype ne $filetype) {
        dbugout(2, "TESTSTYLE: best type is now [ $filetype ]") if $debug;
        $besttype = $filetype;
        %possibles = ();
      }
      dbugout(2, "TESTSTYLE: keeping $filename") if $debug;
      $possibles{$filename} = 1;
    } elsif ($filetype eq 'English text') {
      if ($besttype ne 'ASCII text') {
        if ($besttype ne 'English text') {
          dbugout(2, "TESTSTYLE: best type is now [ $filetype ]")
            if $debug;
          $besttype = $filetype;
          %possibles = ();
        }
        dbugout(2, "TESTSTYLE: keeping $filename") if $debug;
        $possibles{$filename} = 1;
      }
    }
  }

  @candidates = keys %possibles;
  dbugout(2, "TESTSTYLE: remaining candidates: ", join (' <-> ', @candidates))
    if $debug;
  return unless @candidates;
  if ($#candidates == 0) {
    dbugout(2, "TESTSTYLE: SUCCESS with $candidates[0]") if $debug;
    return $candidates[0];
  }

  %possibles = ();
  foreach $filename (@candidates) {
    dbugout(2, "TESTSTYLE: looking for {} in $filename") if $debug;
    $text = slurpfile($filename);
    if ($text =~ /\{.+\}/s) {
      if ($filename =~ /\/Themes\//) {
        dbugout(2, "TESTSTYLE: SUCCESS with $filename (in /Themes/)")
          if $debug;
        return $filename;
      } else {
        dbugout(2, "TESTSTYLE: found {}; keeping $filename") if $debug;
        $possibles{$filename} = 1;
      }
    }
  }
  @candidates = keys %possibles;
  if ($#candidates == 0) {
    dbugout(2, "TESTSTYLE: SUCCESS with $filename (only one with {})")
      if $debug;
    return $candidates[0];
  } elsif ($#candidates > 0) {
    dbugout(2, "TESTSTYLE: multiple possibilities remain, returning first")
     if $debug;
    print "$programname: warning, difficulty identifying theme's style file\n";
    return $candidates[0];
  }
  dbugout(1, "TESTSTYLE: FAILED") if $debug;
  undef;
}

  ###
  #
  #  WM_QUERYDL
  #
  ###

sub wm_querydl {
  my $themetext = shift;

  return query_themes_org($downloadsite, $themetext);
}

  ###
  #
  #  WM_ACTIVATETHEME
  #
  ###

sub wm_activatetheme {
  my $theme = shift;

  forktick("setstyle \"$avthemes{$theme}[0]\"");
}

  ###
  #
  #  WM_UNINSTALLTHEME
  #
  ###

sub wm_uninstalltheme {
  my $theme = shift;

  return dirwipe($avthemes{$theme}[0], 1);
}

  ###
  #
  #  WM_RENAME
  #
  ###

sub wm_rename {
  my ($old, $new) = @_;

  my $oldpath = $avthemes{$old}[0];
  my $newpath = $oldpath;

  if (-d $oldpath) {
    $newpath =~ s/\/\Q$old\E\.themed$/\/$new.themed/ or
      choke("internal error: \"$oldpath\" dir doesn't end in \"$old.themed\"");
  } elsif ($newpath =~ s/\/\Q$old\E$/\/$new/) {
  } elsif ($newpath =~ s/\/\Q$old\E\.style$/\/$new.style/) {
  } else {
    choke("internal error: \"$oldpath\" doesn't end in \"$old\" or \"$old.style\"");
  }

  rename $oldpath, $newpath
    or choke("can't rename $oldpath to $newpath: $!");

  $newpath;
}

  ###
  #
  #  WM_VERSIONOK
  #
  ###

sub wm_versionok {
  my $request = shift;
  my $rqver;

  if ($request =~ /(\d*\.\d+)/) {
    $rqver = $1;
  } else {
    dbugout(2, "WM_VERSIONOK: Can't comprehend theme version \"$request\"!")
      if $debug;
    return 1;
  }

  dbugout(2, "WM_VERSIONOK: deciding about theme for version $rqver")
    if $debug;
  getversion_wmaker();

  if ($rqver < 0.7) {
    return $wmaker_version < 0.7;
  } elsif ($wmaker_version < 0.7) {
    return 0;
  } elsif ($rqver < 1) {
    return $wmaker_version < 1;
  } elsif ($wmaker_version < 1) {
    return 0;
  } else {
    return 1;
  }
}

sub getversion_wmaker {
  if (!defined $wmaker_version) {
    if (`$wm_executable --version` =~ /\s(\d*\.\d+)/) {
      $wmaker_version = $1;
      dbugout(1,
        "GETVERSION_WMAKER: Window Maker's version is $wmaker_version")
        if $debug;
    } else {
      dbugout(1, "Can't get Window Maker's version") if $debug;
    }
  }
  $wmaker_version;
}

1;

