#  Config file/user-specific data handleing for wmtheme

##############################################################
#
#  READ_USER_CONFIG
#
#  Sets global variables  %config_wm
#
#  Call whether or not the configfile exists, it will set
#  default config values
#

sub wmconf_read_userconfig {
  my $configfile = shift;
  my ($line, $count, $setting, $invocation, $value);

  my $yesno_defaultable_configs = join('|', qw(
    confirm_first_backout
    set_favorite_on_backout
    set_queried_backout
    ));

  my %config_default = (
    'confirm_first_backout', 'yes',
    'set_favorite_on_backout', 'no',
    'set_queried_backout', 'no'
    );

  if (! open (CONF, $configfile)) {
    foreach (keys %config_default) {
      $config_wm{$_} = $config_default{$_} if not exists $config_wm{$_};
    }
    return;
  }

  $count = 0;
  while ($line = <CONF>) {
    ++$count;
    chomp $line;
    next if $line =~ /^\s*(#|$)/;
    if ($line =~ /^\s*(\S+)\s+(\S+)\s+(\S+)\s*$/) {
      $setting = lc $1;
      $invocation = lc $2;
      next if $invocation ne $programname;
      $value = $3;
    } elsif ($line =~ /^\s*(\S+)\s+(\S+)\s*$/) {
      $setting = lc $1;
      $invocation = '';
      $value = $2;
    } else {
      choke("malformed setting \"$line\" on line $count of $configfile");
    }

    if ($setting eq 'debug_save') {
      warn "$programname: setting \"debug_save\" is deprecated! Please remove it.\n";
      next;
    }

    if ($setting =~ s/^default_(.+)/$1/i) {
      if ($invocation) {
        choke("$setting takes just one value, on line $count of $configfile");
      }
    } elsif (!$invocation) {
      choke("$setting requires an invocation name, on line $count of $configfile");
    }

    if ($setting =~ /^($yesno_defaultable_configs)$/i) {
      $value = lc $value;
      if ($value =~ /^(yes|no)$/) {
        if ($invocation) {
          $config_wm{$setting} = $value;
        } else {
          $config_default{$setting} = $value;
        }
      } else {
        choke("yes or no is required, on line $count of $configfile");
      }
     # Additional non yes/no settings can be checked for here
    } else {
      choke("unknown setting on line $count of $configfile:", $line);
    }
  }
  close CONF;
  foreach (keys %config_default) {
    $config_wm{$_} = $config_default{$_} if not exists $config_wm{$_};
  }
}

##############################################################
#
#  SETUP_WMTHEME
#
#  Default configuration file installation for wmtheme
#

sub setup_wmtheme {
  my $answer;

  if (-d $wmthemedir) {
    print "It seems wmtheme has already been installed for this user\n";
    print "($wmthemedir exists).  Do you want to overwrite your current\n";
    print "settings with the default settings? [y/N] ";
    $answer = <STDIN>;
    if ($answer !~ /^y/i) {
      print "$programname: setup aborted.\n";
      return;
    }
  }

  if (! -d $wmthemedir) {
    mkdir $wmthemedir, 0755 or
      choke("can't mkdir $wmthemedir: $!");
  }
  if (-f "$sharedir/conf.default") {
    copyfile("$sharedir/conf.default", "$wmthemedir/conf");
  } elsif (-f "$programdir/conf.default") {
    copyfile("$programdir/conf.default", "$wmthemedir/conf");
  } else {
    print "$programname: Your kit is incomplete.  A sample configuration file could\n";
    print "$programname: not be found.\n";
    return;
  }
  stowfile($wmthemelog, ' ');
  print "$programname: Ok, wmtheme has been set up for this user.\n";
  print "  You may want to edit $wmthemedir/conf now.\n";
}

##############################################################
#
#  READ_LOGFILE
#

sub wmconf_read_logfile {
  my $logfilename = shift;
  my %tmprec;

  %loginfo = (
    'lastaction', '',
    'favorite', ''
    );

  open(F, $logfilename) or return;

  while (<F>) {
    if (/^$programname\s+(\S+)\s(.+)$/
   
     #################################################################
     ##
     ##  Semi-temporary code to read old Sawmill data
     ##

        or ($programname eq 'sftheme' and /^smtheme\s+(\S+)\s(.+)$/)

     ##
     #################################################################
    
    ) {
      if ($1 eq 'themerec') {
        %tmprec = decodecgiline($2);
        if ($tmprec{'name'}) {
          foreach (keys %tmprec) {
            $themerecs{$tmprec{'name'}}{$_} = $tmprec{$_};
          }
        }
      } elsif ($1 eq 'installed') {
        push @installlog, $2;
      } else {
        $loginfo{$1} = $2;
      }
    }
  }
  close F;
}

##############################################################
#
#  WRITE_LOGFILE
#

sub write_logfile {
  my ($logtmp, $logfilename) = @_;
  my ($recstr, $theme);

  open(LTMP, ">$logtmp") or
    choke("can't write $logtmp: $!");

  if (open LOG, $logfilename) {
    while (<LOG>) {
      if (/^(\S+)/ and $1 ne $programname

     #################################################################
     ##
     ##  Semi-temporary code to skip old Sawmill data
     ##

          and $1 ne 'smtheme'

     ##
     #################################################################

      ) {
        print LTMP;
      }
    }
    close LOG;
  }

  foreach $theme (keys %themerecs) {
    print LTMP "$programname themerec ", makecgiline(%{$themerecs{$theme}}),
      "\n" if $avthemes{$theme};
  }

  foreach (@installlog) {
    print LTMP "$programname installed $_\n";
  }

  foreach (keys %loginfo) {
    print LTMP "$programname $_ $loginfo{$_}\n"
      if $loginfo{$_};
  }

  close LTMP or choke("error closing $logtmp: $!");

  if (-f $logfilename) {
    unlink $logfilename or
      choke("can't unlink $logfilename: $!");
  }
  rename $logtmp, $logfilename or
    choke("can't rename $logtmp to $logfilename: $!");

}

1;

